<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class Lodgement_control extends CI_Controller {
  
  public function __construct()
  {
    parent::__construct();

    if ($this->session->userdata('prosc_admin') !== true)
      redirect(base_url(), 'refresh');
    $this->load->model('lodgement_model');
  }

  public function index()
  {
    $this->pager();
  }

  function pager($tmp='')
  {
    if (!$this->input->is_ajax_request() && strstr(current_url(), '.aspx') == '') redirect(base_url('not_found.aspx'), 'refresh');
    $data = $this->lodgement_model->_get_all() ?: [];
    $users = $this->lodgement_model->_get_users();
    $clients = $this->lodgement_model->_get_clients();
    $outlets = $this->lodgement_model->_get_outlets();
    if ($tmp == '') $this->load->view('default', ['page'=>'lodgements', 'data'=>$data, 'users'=>$users, 'clients'=>$clients, 'outlets'=>$outlets, 'tmp'=>$tmp]);
    else return $this->load->view('admin/lodgements', ['data'=>$data, 'users'=>$users, 'clients'=>$clients, 'outlets'=>$outlets, 'tmp'=>$tmp], true);
  }

  function check()
  {
    $name = trim($this->input->post('name'));
    $date = trim($this->input->post('dated'));
    $amnt = trim($this->input->post('amount'));

    if ($name == '' OR $amnt == '' OR !is_numeric($amnt) OR $date == '')
      return false;

    if ($date > date('Y-m-d')) return ['status'=>'error', 'message'=>'Payment date is invalid'];

    $supply = $this->lodgement_model->_get_supply($name);
    if ( ! $supply) return ['status'=>'error', 'message'=>'Supply Voucher: <b class="text-warning">'. $name .'</b> is invalid'];

    // Log Changes ...
    return [
      'voucher'  => $name,
      'date'     => $date,
      'amount'   => $amnt,
      'op_date'  => $supply->date,
      'voucher'  => $supply->voucher,
      'customer' => $supply->customer,
      'done_by'  => $_SESSION['prosc_user']
    ];
  }

  function create()
  {
    $data = $this->check();
    if ( ! $data) exit(json_encode(['status'=>'error', 'message'=>'Some fields were not properly filled']));
    if (array_key_exists('status', $data)) exit(json_encode($data));
    if ( ! $this->lodgement_model->_post($data))
      exit(json_encode(['status'=>'error', 'message'=>'An unknown error occured and the insertion operation failed']));

    echo (json_encode(['status'=>'success', 'message'=>$this->pager('dump')]));
  }

  function update()
  {
    $target = trim($this->input->get('target'));
    $old_data = $this->lodgement_model->_get(['token'=>$target]);
    if ( ! $old_data) exit(json_encode(['status'=>'error', 'message'=>'A fatal error has occured. <br />Please refresh and try again']));
    $data = $this->check();
    if (array_key_exists('status', $data)) exit(json_encode($data));
    if ( ! $data) exit(json_encode(['status'=>'error', 'message'=>'Some fields were not properly filled']));
    if ( ! $this->lodgement_model->_put($data, $target))
      exit(json_encode(['status'=>'error', 'message'=>'An unknown error occured and the update failed']));
    $this->load->model('debtor_model');

    $supplies = $this->debtor_model->_sum_supply($data['customer']);
    $payments = $this->debtor_model->_sum_payment($data['customer']);
    $amount = $payments - $supplies;
    $this->debtor_model->_put(['wallet'=>$amount], $data['customer']);

    $supplies = $this->debtor_model->_sum_supply($old_data->customer);
    $payments = $this->debtor_model->_sum_payment($old_data->customer);
    $amount = $payments - $supplies;
    $this->debtor_model->_put(['wallet'=>$amount], $old_data->customer);

    echo (json_encode(['status'=>'success', 'message'=>$this->pager('dump')]));
  }

  function delete()
  {
    $target = trim($this->input->get('token'));
    if ( ! $this->lodgement_model->_get(['token'=>$target])) exit(json_encode(['status'=>'error', 'message'=>'A fatal error has occured. <br />Please refresh and try again']));

    $this->lodgement_model->_drop($target);
    echo (json_encode(['status'=>'success', 'message'=>$this->pager('dump')]));
  }

  function popup()
  {
    if ( ! $this->input->is_ajax_request()) {
      $pagedata['heading'] = 'Hold on!';
      $pagedata['message'] = 'The page you tried to reach, restricts direct access.';
      $this->load->view('errors/html/error_404', $pagedata);
    }
    $target = $this->input->get('token');
    $last = $this->lodgement_model->_count() + 1;
    $data = ['data'=>'', 'last'=>$last];
    $file = $this->lodgement_model->_get(['token'=>$target]);
    $users = $this->lodgement_model->_get_users();
    $outlets = $this->lodgement_model->_get_outlets();
    if ($file) $data['data'] = $file;
    $data['users'] = $users;
    $data['outlets'] = $outlets;
    $this->load->view('popups/admin/lodgement', $data);
  }
}
