<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Outlet_control extends CI_Controller
{

  public function __construct()
  {
    parent::__construct();

    if ($this->session->userdata('prosc_admin') !== true)
      redirect(base_url(), 'refresh');

    $this->load->model('outlet_model');
    $this->load->model('shipping_settings_model');
  }

  public function index()
  {
    $this->pager();
  }

  function pager($tmp = '')
  {
    if (!$this->input->is_ajax_request() && strstr(current_url(), '.aspx') == '') redirect(base_url('not_found.aspx'), 'refresh');
    $this->db->order_by('name');
    $data = $this->outlet_model->_get_all() ?: [];
    $country = $this->shipping_settings_model->_get_country();

    if ($country->value === 'Nigeria') {
      $states = $this->shipping_settings_model->_get_nigeria() ?:
        $s = [];
    } else {
      $states = $this->shipping_settings_model->_get_uk() ?: $s = [];
    }
    if ($tmp == '') $this->load->view('default', ['page' => 'outlet', 'data' => $data, 'states' => $states, 'tmp' => $tmp]);
    else return $this->load->view('admin/outlet', ['data' => $data, 'states' => $states, 'tmp' => $tmp], true);
  }

  function check()
  {
    $name  = trim($this->input->post('name')) ?: trim($this->input->post('autogen'));
    $addr  = trim($this->input->post('address'));
    $state = trim($this->input->post('state'));
    if ($name == '' or $addr == '' or $state == '') return false;
    if ($this->outlet_model->_get(['name' => $name, 'state' => $state])) return 'This outlet already exist';
    if (! ctype_alnum(str_replace(' ', '', $name))) return 'Outlet name cannot contain special characters';
    if (is_numeric($name)) return 'Outlet name cannot be a number';

    return [
      'name'    => $name,
      'type'    => 1,
      'address' => $addr,
      'state'   => $state
    ];
  }

  function create()
  {
    $data = $this->check();
    if (! $data) exit(json_encode(['status' => 'error', 'message' => 'Fill in all fields']));
    if (! is_array($data)) exit(json_encode(['status' => 'error', 'message' => $data]));
    if (! $this->outlet_model->_post($data))
      exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured and the insertion operation failed']));

    echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
  }

  function update()
  {
    $target = trim($this->input->get('target'));
    if (! $this->outlet_model->_get(['token' => $target])) exit(json_encode(['status' => 'error', 'message' => 'A fatal error has occured. <br />Please refresh and try again']));
    $data = $this->check();
    if (! $data) exit(json_encode(['status' => 'error', 'message' => 'Fill in all fields']));
    if (! is_array($data)) exit(json_encode(['status' => 'error', 'message' => $data]));

    if (! $this->outlet_model->_put($data, $target))
      exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured and the update failed']));

    echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
  }

  function delete()
  {
    $target = trim($this->input->get('token'));
    if (! $this->outlet_model->_get(['token' => $target])) exit(json_encode(['status' => 'error', 'message' => 'A fatal error has occured. <br />Please refresh and try again']));

    $this->outlet_model->_drop($target);
    $this->session->set_flashdata('prosc_message', 'Record Deleted');
    echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
  }

  function popup()
  {
    if (! $this->input->is_ajax_request()) {
      $pagedata['heading'] = 'Hold on!';
      $pagedata['message'] = 'The page you tried to reach, restricts direct access.';
      $this->load->view('errors/html/error_404', $pagedata);
    }
    $last = $this->outlet_model->_count() + 1;
    $target = $this->input->get('token');
    $data = ['data' => '', 'last' => $last];
    $file = $this->outlet_model->_get(['token' => $target]);
    $country = $this->shipping_settings_model->_get_country();

    if ($country->value === 'Nigeria') {
      $states = $this->shipping_settings_model->_get_nigeria() ?:
        $s = [];
    } else {
      $states = $this->shipping_settings_model->_get_uk() ?:
        $s = [];
    }
    if ($file) $data['data'] = $file;
    $data['states'] = $states;
    $this->load->view('popups/admin/outlet', $data);
  }
}
