<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Sales_control extends CI_Controller
{

  public function __construct()
  {
    parent::__construct();

    if (!isset($_SESSION['prosc_user']))
      redirect(base_url(), 'refresh');

    $this->load->model('sales_model');
    $this->load->model('inventory_model');
    $this->load->model('dashboard_model');
    $this->load->model('shipping_settings_model');
  }

  public function index()
  {
    $this->pager();
  }

  function pager($tmp = '')
  {
    if (!$this->input->is_ajax_request() && strstr(current_url(), '.aspx') == '') redirect(base_url('not_found.aspx'), 'refresh');
    if ($this->input->get('action') == 'return') return $this->returns($tmp);
    $data['tmp']    = $tmp;
    $data['page']   = 'sales';
    $data['cats']   = $this->sales_model->_get_cats() ?: [];
    $data['data']   = $this->sales_model->_get_all() ?: [];
    $data['items']  = $this->sales_model->_get_items() ?: [];
    $data['datax']  = $this->sales_model->_get_allx() ?: [];
    $data['sales']  = $this->sales_model->_get_sales('0', $this->input->get('token')) ?: [];
    $data['orders'] = $this->sales_model->_get_sales('3', $this->input->get('token')) ?: [];
    $data['return'] = $this->sales_model->_get_return() ?: [];
    $data['totals'] = $this->sales_model->_get_total($this->input->get('token')) ?: [];
    $data['stocks'] = $this->sales_model->_get_stock() ?: [];

    $country = $this->shipping_settings_model->_get_country();

    if ($country->value === 'Nigeria') {
      $data['states'] = $this->shipping_settings_model->_get_nigeria() ?: [];
    } else {
      $data['states'] = $this->shipping_settings_model->_get_uk() ?: [];
    }
    $data['outlet'] = $this->sales_model->_get_outlet($this->input->get('token')) ?: [];
    $data['payment'] = $this->sales_model->_get_payments() ?: [];
    $data['currency'] = $this->dashboard_model->_get_currency() ?: "";

    if ($tmp == '') $this->load->view('default', $data);
    else return $this->load->view('admin/sales', $data, true);
  }

  function returns($tmp = '')
  {
    if (!$this->input->is_ajax_request() && strstr(current_url(), '.aspx') == '') redirect(base_url('not_found.aspx'), 'refresh');
    $data['tmp']    = $tmp;
    $data['page']   = 'returns';
    $data['cats']   = $this->sales_model->_get_cats() ?: [];
    $data['sale']   = $this->sales_model->_get(['token' => $this->input->get('token')]);
    $data['data']   = $this->sales_model->_get_all_returns($this->input->get('token'));
    $data['items']  = $this->sales_model->_get_items() ?: [];
    if ($tmp == '') $this->load->view('default', $data);
    else return $this->load->view('admin/returns', $data, true);
  }

  function check_detail($token = '')
  {
    // $dated  = $dated ?: trim($this->input->post('dated'));
    $client = trim($this->input->post('customer'));
    $byname = trim($this->input->post('name'));
    $bphone = trim($this->input->post('phone'));
    $email  = trim($this->input->post('email'));
    $ctype  = trim($this->input->post('c-type'));
    $ptype  = trim($this->input->post('pay-type'));
    $pspan  = trim($this->input->post('pay-span'));
    $amount = trim($this->input->post('pay-amount'));

    if (! in_array($ctype, ['walk-in', 'special'])) exit(json_encode(['status' => 'error', 'message' => 'Customer type unavailable']));

    if ($ctype == 'walk-in') {
      if ($byname == '' or $bphone == '') exit(json_encode(['status' => 'error', 'message' => 'Fill in all fields']));
      if (! ctype_alpha(str_replace(' ', '', $byname))) exit(json_encode(['status' => 'error', 'message' => 'Enter a valid name']));
      if (! is_numeric(substr($bphone, 1)) or strlen($bphone) < 11 or ! in_array(substr($bphone, 0, 1), ['+', '0']))
        exit(json_encode(['status' => 'error', 'message' => 'Phone number is invalid']));
      if ($email && ! filter_var($email, FILTER_VALIDATE_EMAIL)) exit(json_encode(['status' => 'error', 'message' => 'Email is invalid']));
    } else {
      $customer = $this->sales_model->_get_client($client);
      if (! $customer) exit(json_encode(['status' => 'error', 'message' => 'Select customer from the list']));
      if (! in_array($ptype, ['paynow', 'install'])) exit(json_encode(['status' => 'error', 'message' => 'Payment type unavailable']));
      if ($ptype == 'install' && (! is_numeric($pspan) or $pspan < 1)) exit(json_encode(['status' => 'error', 'message' => 'Enter a valid payment span']));
      // if ( ! is_numeric($amount) OR $amount < 100) exit(json_encode(['status'=>'error', 'message'=>'Enter a valid payment amount']));

      $byname = $customer->name;
      $bphone = $customer->phone;
      $ctype = 'special';
    }
    if ($token == '' && $this->sales_model->_get_keyed($token)) exit(json_encode(['status' => 'error', 'message' => 'Request Error']));

    if ($token != '') $name = $token;
    $count = $this->sales_model->_count() + 1001;
    $count = (strlen($count) < 7 ? str_repeat('0', 7 - strlen($count)) : '') . $count;
    $key = $token ?: $count;
    $_SESSION['prosc_sales_email'] = $email;
    $_SESSION['prosc_sales_span']  = $pspan;

    $type = ($ctype == 'special') ? 1 : 0;
    $this->set_sale($type, $key, date('Y-m-d H:i'), $byname, $bphone);

    $data = ['message' => ''];
    if ($token == '') $data = ['message' => $this->pager('dump'), 'extra' => 'Starting Sale'];
    echo (json_encode(array_merge($data, ['status' => 'success'])));
  }

  function set_sale($type, $voucher, $date, $name, $phone)
  {
    $_SESSION['prosc_sales_dated'] = $date;
    $_SESSION['prosc_sales_type'] = $type;
    $_SESSION['prosc_sales_key'] = $voucher;
    $_SESSION['prosc_sales_name'] = $name;
    $_SESSION['prosc_sales_phone'] = $phone;

    return true;
  }

  function unsess()
  {
    unset($_SESSION['prosc_sales_key']);
    unset($_SESSION['prosc_sales_type']);
    unset($_SESSION['prosc_sales_name']);
    unset($_SESSION['prosc_sales_dated']);
    unset($_SESSION['prosc_sales_phone']);

    echo json_encode(['status' => 'success', 'message' => '']);
  }

  function navigate($dir)
  {
    if (($dir != 'next' && $dir != 'prev')) return false;

    $chip = $dir == 'prev' ? '-1 days' : '+1 day';
    $dated = $this->session->userdata('prosc_sales_date') ?? date('Y-m-d');
    $dated = date('Y-m-d', strtotime($chip, strtotime($dated)));
    $this->session->set_userdata('prosc_sales_date', $dated);
    echo json_encode(['status' => 'success', 'message' => '']);
  }

  function check()
  {
    $item = trim($this->input->post('item'));
    $qty  = trim($this->input->post('quantity'));
    $price1 = $this->input->post('manual_price');
    $checkX = $this->input->post('setman');

    if ($item == '' or !is_numeric($qty)) return false;
    if ($qty < 1) return -1;
    if ($checkX && (!is_numeric($price1) or $price1 < 1)) return -2;

    $price = $this->sales_model->_get_price($item, 'a577f519b58b533e400cf28a7464576babf92121');

    return [
      'span'             => $_SESSION['prosc_sales_span'],
      'warehouse'        => $_SESSION['prosc_warehouse'],
      'date'             => $_SESSION['prosc_sales_dated'],
      'type'             => $_SESSION['prosc_sales_type'],
      'item'             => $item,
      'price'            => (! $checkX ? $price[0] : $price1),
      'total'            => $qty * (! $checkX ? $price[0] : $price1),
      'voucher'          => $_SESSION['prosc_sales_key'],
      'cost_price'       => $price[1],
      'total_cost_price' => $qty * $price[1],
      'quantity'         => $qty,
      'done_by'          => $_SESSION['prosc_user']
    ];
  }

  function create()
  {
    if ($this->input->get('target') == 'returns') return $this->check_return();
    if ($this->session->userdata('prosc_sales_key') == '') return $this->check_detail();

    $data = $this->check();
    if (! is_array($data)) exit(json_encode(['status' => 'error', 'message' => 'Some fields were not properly filled']));
    if ($data == -1) exit(json_encode(['status' => 'error', 'message' => 'Quantity is invalid']));
    if ($data == -2) exit(json_encode(['status' => 'error', 'message' => 'Enter a valid price']));
    $check = ['item' => $data['item'], 'voucher' => $data['voucher']];
    if ($this->sales_model->_get($check)) exit(json_encode(['status' => 'error', 'message' => 'This entry already exists in the sale']));

    $this->inventory_model->_aggregate($_SESSION['prosc_warehouse']);

    if (! $this->sales_model->_evaluate($data['item'], $data['quantity']))
      exit(json_encode(['status' => 'error', 'message' => 'Insufficient Stock']));

    if (! $this->sales_model->_post($data))
      exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured and the insertion operation failed']));

    echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
  }

  function update()
  {
    $action = $this->input->get('action');
    if ($action != '') exit($this->$action());
    $target = $this->sales_model->_get(['token' => trim($this->input->get('target'))]);
    if (! $target) exit(json_encode(['status' => 'error', 'message' => 'A fatal error has occured. <br />Please refresh and try again']));
    $data = $this->check();
    if (! $data) exit(json_encode(['status' => 'error', 'message' => 'Some fields were not properly filled']));
    if ($data == -1) exit(json_encode(['status' => 'error', 'message' => 'Quantity is invalid']));
    if ($data == -2) exit(json_encode(['status' => 'error', 'message' => 'Enter a valid price']));
    $check = ['token !=' => $target->token, 'item' => $data['item'], 'voucher' => $data['voucher']];
    if ($this->sales_model->_get($check)) exit(json_encode(['status' => 'error', 'message' => 'This entry already exists in the sale']));

    $this->inventory_model->_aggregate($_SESSION['prosc_warehouse']);
    $quantity = $data['quantity'] - $target->quantity;
    if (! $this->sales_model->_evaluate($data['item'], $quantity))
      exit(json_encode(['status' => 'error', 'message' => 'Insufficient Stock']));

    if (! $this->sales_model->_put($data, $target->token))
      exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured and the update failed']));

    echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
  }

  function check_return()
  {
    if ($this->input->get('extras') == 'account') $this->account();

    $item     = trim($this->input->post('item'));
    $reason   = trim($this->input->post('reason'));
    $details  = trim($this->input->post('details'));
    $quantity = trim($this->input->post('quantity'));

    if (! in_array($reason, ['faulty', 'wrong', 'late']) or $item == '' or $quantity == '')
      exit(json_encode(['status' => 'error', 'message' => 'Fill in all fields']));
    $sale = $this->sales_model->_get(['voucher' => $this->input->get('voucher')]);
    if (! $sale) exit(json_encode(['status' => 'error', 'message' => 'A fatal error occurred. Please refresh and try again']));
    $return = $this->sales_model->_get_return_item($this->input->get('voucher'), $item);
    if (! is_numeric($quantity) or $quantity < 1) exit(json_encode(['status' => 'error', 'message' => 'Enter a valid quantity']));
    if ($return - $quantity < 0) exit(json_encode(['status' => 'error', 'message' => "Maximum return quantity for this item is: &nbsp; " . number_format($return)]));

    $data = [
      'customer' => $sale->customer,
      'voucher'  => $sale->voucher,
      'item'     => $item,
      'category' => $reason,
      'reason'   => $details,
      'price'    => $sale->price,
      'total'    => $sale->price * $quantity,
      'quantity' => $quantity
    ];
    $this->sales_model->_post($data, 'returns');
    exit(json_encode(['status' => 'success', 'message' => $this->returns('dump')]));
  }

  function open()
  {
    $target = trim($this->input->get('target'));
    $sale = $this->sales_model->_get(['token' => $target]);
    $debtor = $this->sales_model->_get_client(['token' => $sale->customer ?? '']);
    if (! $sale or ! $debtor) exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured while trying to open']));

    if (! $this->set_sale($sale->type, $sale->voucher, $sale->date, $sale->customer_name, $sale->customer_phone))
      exit(json_encode(['status' => 'error', 'message' => 'Failed to include sale data']));

    echo json_encode(['status' => 'success', 'message' => '']);
  }

  function mod()
  {
    // $dated  = trim($this->input->post('dated'));
    $ctype = trim($this->input->post('c-type'));
    $ptype = trim($this->input->post('pay-type'));
    $pspan = trim($this->input->post('pay-span'));
    $byname = trim($this->input->post('name'));
    $bphone = trim($this->input->post('phone'));
    $bemail = trim($this->input->post('email'));

    if (! in_array($ctype, ['walk-in', 'special'])) exit();
    if ($ctype == 'walk-in') {
      if ($byname == '' or $bphone == '') exit(json_encode(['status' => 'error', 'message' => 'Fill in all fields']));
      // if ($dated > date('Y-m-d')) exit(json_encode(['status'=>'error', 'message'=>'Invalid date supplied']));
      if (! is_numeric(substr($bphone, 1)) or strlen($bphone) < 11 or ! in_array(substr($bphone, 0, 1), ['+', '0']))
        exit(json_encode(['status' => 'error', 'message' => 'Phone number is invalid']));
      if ($bemail && ! filter_var($bemail, FILTER_VALIDATE_EMAIL)) exit(json_encode(['status' => 'error', 'message' => 'Email is invalid']));
    }
    if ($ptype == 'install' && $pspan < 1) exit(json_encode(['status' => 'error', 'message' => 'Enter a valid payment span']));
    $client = $this->sales_model->_get_client(['name' => $byname, 'phone' => $bphone]);
    if ($ctype == 'special') {
      $client = $this->sales_model->_get_client($this->input->post('customer'));

      if (! $client) exit(json_encode(['status' => 'error', 'message' => 'Select a customer from the list']));
    }

    $sale = $this->sales_model->_get(['voucher' => $this->input->get('target')]);
    if (! $sale) exit(json_encode(['status' => 'error', 'message' => 'A fatal error has occured. <br />Please refresh and try again']));

    // $check = ['date'=>$dated, 'customer'=>$client, 'customer_name'=>$byname, 'customer_phone'=>$bphone];
    // if ($this->sales_model->_get($check)) exit(json_encode(['status'=>'error', 'message'=>'This Sale already exist']));
    if (! $client) {
      $this->load->model('debtor_model');
      $cdata = [
        'name'  => $byname,
        'phone' => $bphone,
        'email' => $bemail,
        'type'  => 1,
        'password' => password_hash('XC@Customer', PASSWORD_DEFAULT)
      ];
      $debtor = $this->debtor_model->_post($cdata);
      $client = $this->sales_model->_get_client($debtor);
    }

    if (! is_numeric($pspan) or $pspan == 0 or $ptype != 'install') {
      $pspan = 0;
      $data = $this->paynow($sale->voucher, 0);
    }

    $data['type'] = $ctype == 'special' ? 1 : 0;
    // if (array_key_exists('span', $data)) $pspan = 0;
    if (is_numeric($pspan) && $pspan > 0) {
      $this->load->model('payment_model');
      $this->payment_model->_drop(['supply_id' => $sale->voucher]);
    }
    $data['span'] = $pspan != '' ? $pspan : 0;
    $data['customer'] = $client->token;
    $data['customer_name'] = $client->name;
    $data['customer_phone'] = $client->phone;
    $this->sales_model->_put($data, ['voucher' => $sale->voucher]);
    echo (json_encode(['status' => 'success', 'message' => '', 'extra' => 'reload']));
  }

  function close()
  {
    $cname = trim($this->session->userdata('prosc_sales_name'));
    $phone = trim($this->session->userdata('prosc_sales_phone'));
    $email = trim($this->session->userdata('prosc_sales_email'));

    $voucher = trim($this->session->userdata('prosc_sales_key'));
    if (! $this->sales_model->_get_keyed($voucher)) {
      $this->session->set_flashdata('prosc_message', 'No Sales data found. Operation aborted...');
      exit(json_encode(['status' => 'success', 'message' => '']));
      unset($_SESSION['prosc_sales_key']);
    }

    // $_SESSION['prosc_sales_span'] = $ptype == 'install' ? $pspan : -1;
    // $_SESSION['prosc_sales_name'] = $byname;
    // $_SESSION['prosc_sales_type'] = (($ctype == 'walk-in') ? 0 : 1);
    // $_SESSION['prosc_sales_phone'] = $bphone;
    // $_SESSION['prosc_sales_amount'] = $amount;

    $customer = $this->sales_model->_get_client(['name' => $cname, 'phone' => $phone]);
    if (! $customer) {
      $this->load->model('debtor_model');
      $cdata = [
        'name'  => $cname,
        'phone' => $phone,
        'email' => $email,
        'type'  => 1,
        'password' => password_hash('XC@Customer', PASSWORD_DEFAULT)
      ];
      $debtor = $this->debtor_model->_post($cdata);
      $customer = $this->sales_model->_get_client($debtor);
    }

    if ($this->input->get('as-order') == 'true') {
      $this->load->model('payment_model');
      $this->payment_model->_drop(['supply_id' => $voucher]);
      $data['type'] = 3;
      // $data['span'] = 0;
    } else {
      $data = $this->paynow($voucher);
      $data['type'] = $customer->type == 1 ? 0 : 1;
    }

    $data['customer'] = $customer->token;
    $data['customer_name'] = $customer->name;
    $data['customer_phone'] = $customer->phone;

    $this->sales_model->_close($data);
    unset($_SESSION['prosc_sales_key']);
    echo (json_encode(['status' => 'success', 'message' => '']));
  }

  function paynow($voucher, $span = -1)
  {
    $sale = $this->sales_model->_get_sale($voucher);
    if (($sale && $sale['span'] == 0) or $span == 0) {
      $this->load->model('payment_model');
      $pay = $this->payment_model->_get(['supply_id' => $voucher]);
      if (! $pay) {
        $last = $this->payment_model->_count() + 800001;
        $payment = [
          'supply_id' => $sale['voucher'],
          'date'      => $sale['date'],
          'pid'       => 'PAE' . $last,
          'paid'      => 1
        ];
      }
      $payment['amount'] = $sale['total'];
      if (! $pay) $this->payment_model->_post($payment, $sale['customer']);
      else $this->payment_model->_put($payment, $pay->token);
    } else return [];

    return ['paid' => 1];
  }

  function delete()
  {
    $token = 'token';
    $target = trim($this->input->get('token'));
    $item = $this->sales_model->_get(['token' => $target]);
    if (! $item) {
      $token = 'voucher';
      $target = trim($this->input->get('voucher'));
      $item = $this->sales_model->_get(['voucher' => $target]);
    }

    if (! $item) exit(json_encode(['status' => 'error', 'message' => 'A fatal error has occured. <br />Please refresh and try again']));

    $this->sales_model->_drop($target, $token);
    $this->session->set_flashdata('prosc_message', 'Record Deleted');
    $msg = ($token == 'voucher') ? '' : $this->pager('dump');
    echo (json_encode(['status' => 'success', 'message' => $msg]));
    $this->inventory_model->_aggregate($_SESSION['prosc_warehouse']);
  }

  function popup()
  {
    $this->session->unset_userdata('prosc_sales_key');
    if (! $this->input->is_ajax_request()) {
      $pagedata['heading'] = 'Hold on!';
      $pagedata['message'] = 'The page you tried to reach, restricts direct access.';
      $this->load->view('errors/html/error_404', $pagedata);
    }
    $target = $this->input->get('token');
    $last = $this->sales_model->_count() + 1001;
    $last = str_repeat('0', 7 - strlen($last)) . $last;
    $data['data']    = [];
    $data['voucher'] = $last;
    $this->db->where('type', 0);
    $data['clients'] = $this->sales_model->_get_clients();
    $file = $this->sales_model->_get(['token' => $target]);
    if ($file) $data['data'] = $file;
    $this->load->view('popups/admin/sale', $data);
  }
}
