<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Warehouse_control extends CI_Controller
{

  public function __construct()
  {
    parent::__construct();

    if ($this->session->userdata('prosc_admin') !== true)
      redirect(base_url(), 'refresh');

    $this->load->model('warehouse_model');
    $this->load->model('shipping_settings_model');
  }

  public function index()
  {
    $this->pager();
  }

  function pager($tmp = '')
  {
    if (!$this->input->is_ajax_request() && strstr(current_url(), '.aspx') == '') redirect(base_url('not_found.aspx'), 'refresh');
    $this->db->order_by('whs_id');
    $data = $this->warehouse_model->_get_all() ?: [];
    $country = $this->shipping_settings_model->_get_country();

    if ($country->value === 'Nigeria') {
      $states = $this->shipping_settings_model->_get_nigeria() ?: [];
    } else {
      $states = $this->shipping_settings_model->_get_uk() ?: [];
    }
    if ($tmp == '') $this->load->view('default', ['page' => 'warehouses', 'data' => $data, 'states' => $states, 'tmp' => $tmp]);
    else return $this->load->view('admin/warehouses', ['data' => $data, 'states' => $states, 'tmp' => $tmp], true);
  }

  function check()
  {
    $addr  = trim($this->input->post('address'));
    $state = trim($this->input->post('state'));
    $whs_name = trim($this->input->post('whs_name'));

    if ($addr == '' or $state == '' or $whs_name == '')
      return false;

    return [
      'type'     => 0,
      'address'  => $addr,
      'state'    => $state,
      'name' => $whs_name
    ];
  }

  function create()
  {
    $data = $this->check();
    if (!$data) exit(json_encode(['status' => 'error', 'message' => 'Fill in all fields']));

    if (!$this->warehouse_model->_post($data))
      exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured and the insertion operation failed']));

    echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
  }

  function update()
  {
    $target = trim($this->input->get('target'));
    if (!$this->warehouse_model->_get(['token' => $target])) exit(json_encode(['status' => 'error', 'message' => 'A fatal error has occured. <br />Please refresh and try again']));
    $data = $this->check();
    if (!$data) exit(json_encode(['status' => 'error', 'message' => 'Fill in all fields']));

    if (!$this->warehouse_model->_put($data, $target))
      exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured and the update failed']));

    echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
  }

  function delete()
  {
    $target = trim($this->input->get('token'));
    if (!$this->warehouse_model->_get(['token' => $target])) exit(json_encode(['status' => 'error', 'message' => 'A fatal error has occured. <br />Please refresh and try again']));

    $this->warehouse_model->_drop($target);
    $this->session->set_flashdata('prosc_message', 'Record Deleted');
    echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
  }

  function popup()
  {
    if (!$this->input->is_ajax_request()) {
      $pagedata['heading'] = 'Hold on!';
      $pagedata['message'] = 'The page you tried to reach, restricts direct access.';
      $this->load->view('errors/html/error_404', $pagedata);
    }
    $last = $this->warehouse_model->_count(['type' => '0']) + 1;
    $target = $this->input->get('token');
    $data = ['data' => '', 'last' => $last];
    $file = $this->warehouse_model->_get(['token' => $target]);
    $country = $this->shipping_settings_model->_get_country();

    if ($country->value === 'Nigeria') {
      $states = $this->shipping_settings_model->_get_nigeria() ?: [];
    } else {
      $states = $this->shipping_settings_model->_get_uk() ?: [];
    }
    $s = [];
    if ($file) $data['data'] = $file;
    $data['states'] = $states;
    $this->load->view('popups/admin/warehouse', $data);
  }
}
