<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Waybill_control extends CI_Controller
{

    public function __construct()
    {
        parent::__construct();

        if ($this->session->userdata('prosc_warehouse') == "")
            redirect(base_url(), 'refresh');

        $this->load->model('waybill_model');
        $this->load->model('shipping_settings_model');
    }

    public function index()
    {
        $this->pager();
    }

    function pager($tmp = '')
    {
        if (!$this->input->is_ajax_request() && strstr(current_url(), '.aspx') == '') redirect(base_url('not_found.aspx'), 'refresh');

        $data['tmp'] = $tmp;
        $data['file'] = $this->waybill_model->_get(['token' => $this->input->get('target')]);
        $data['page'] = 'waybill';
        $data['waybill_data'] = $this->waybill_model->_get_all() ?:  [];
        $country = $this->shipping_settings_model->_get_country();

        if ($country->value === 'Nigeria') {
            $data['states'] = $this->shipping_settings_model->_get_nigeria() ?: [];
        } else {
            $data['states'] = $this->shipping_settings_model->_get_uk() ?: [];
        }
        if ($tmp == '') $this->load->view('default', ['page' => $data['page'], 'data' => $data]);
        else return $this->load->view("admin/waybill" . $data['page'], $data, true);
    }

    function check()
    {
        $items = $this->input->post('items');
        $name = trim($this->input->post('driverName'));
        $phone = trim($this->input->post('driverPhone'));
        $state = trim($this->input->post('state'));
        $voucher = trim($this->input->post('voucher'));
        $shippingTokens = array();
        $shippingAddress = '';



        if ($name == '' || $phone == '' || $state == '') {
            return 'Operation failed! Please fill in all fields';
        }

        if (!ctype_alpha(str_replace(' ', '', $name))) {
            return 'Driver name is invalid';
        }

        if (!is_numeric($phone) || strlen($phone) != 11) {
            return 'Phone number must be 11 digits';
        }

        // Check if there are any selected items
        if (empty($items)) {
            return 'Operation failed! No valid selections were made';
        }

        foreach ($items as $item) {
            $order   = $this->waybill_model->_get_sales(['voucher' => $voucher, 'item' => $item, 'waybill' => '']);
            $order_xtral = $this->waybill_model->_get_sales_xtal(['voucher' => $voucher], true);
            $dataArray = json_decode($order_xtral->shipping, true);
            $result = array(
                'city' => $dataArray['city'],
                'address' => $dataArray['address'],
                'country' => $dataArray['country'],
                'zipcode' =>  $dataArray['zipcode']
            );
            $shippingAddress = implode(' ', $result);
            $itemDetails = [
                'sales_id' => $order->token,
                'order_id' => $order->voucher,
                'customer' => $order->customer,
                'delivery_address' => implode(' ', $result),
                'item' => $order->item,
                'quantity' => $order->quantity,
                'shipment_status' => 'Shipment Created',
                'status' => 0,
            ];
            if (!$itemDetails) exit(json_encode(['status' => 'error', 'message' => 'Data not found']));
            $result = $this->waybill_model->_post_shipping($itemDetails, 'er_sales');
            if (!$result) exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured and the insertion operation failed']));
            array_push($shippingTokens, $result);
        }

        $data = [
            'items'    => implode(' ', $items),
            'state'     => $state,
            'voucher' => $voucher,
            'phone'    => $phone,
            'driver'   => $name,
            'delivery_address' => $shippingAddress,
            'shipping_token'   => implode(' ', $shippingTokens),
            'delivery' => 0,
        ];
        return $data;
    }

    function deliver($items, $waybillToken, $orderVoucher)
    {

        // Ensure $items is not empty
        if (empty($items)) {
            return 'Operation failed! item empty'; // or throw an exception, or handle error
        }

        // Explode $items into an array
        $itemsArray = explode(' ', $items);

        // Iterate over items and apply modifications
        foreach ($itemsArray as $item) {
            if (!empty($item)) {  // Ensure the item is not empty
                // Update the waybill in sales table
                if (!$this->waybill_model->_mod(['voucher' => $orderVoucher, 'item' => $item, 'waybill' => ''], ['waybill' => $waybillToken])) return 'Operation failed! Error sales';

                // Update the waybill in shipping table
                if (!$this->waybill_model->_mod_shipping(['order_id' => $orderVoucher, 'item' => $item, 'waybill' => ''], ['waybill' => $waybillToken])) return 'Operation failed! Error Shipping';
            }
        }

        return true;
    }


    function create()
    {
        $data = $this->check();
        if (!$data) exit(json_encode(['status' => 'error', 'message' => 'Fill in data']));
        if (!is_array($data)) exit(json_encode(['status' => 'error', 'message' => $data]));
        $token = $this->waybill_model->_post($data);
        if (!$token) exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured and the operation failed']));
        $update = $this->deliver($data['items'], $token,  $data['voucher']);
        if (!$update) exit(json_encode(['status' => 'error', 'message' => $update]));;
        echo json_encode(['status' => 'success', 'message' => base_url('ipanel/waybill.aspx')]);
    }

    function update()
    {
        $token = $this->input->get('token');
        $action = trim($this->input->get('action'));
        if ($action == 'approve') {
            $this->waybill_model->_put(['delivery' => 1], $token);
            exit(json_encode(['status' => 'success', 'message' => base_url("ipanel/waybill.aspx")]));
        }
        $data = $this->check();
        if (!$this->waybill_model->_get(['token' => $token])) exit(json_encode(['status' => 'error', 'message' => 'A fatal error has occured. <br />Please refresh and try again']));
        if (!$data) exit(json_encode(['status' => 'error', 'message' => 'Fill in all fields']));
        if (!is_array($data)) exit(json_encode(['status' => 'error', 'message' => $data]));

        if (!$this->waybill_model->_put($data, $token))
            exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured and the update failed']));

        $this->deliver($data['items'], $token);
        echo (json_encode(['status' => 'success', 'message' => base_url("generate/invoice.aspx?action=waybill&target=$token")]));
    }

    function popup()
    {
        if (!$this->input->is_ajax_request()) {
            $pagedata['heading'] = 'Hold on!';
            $pagedata['message'] = 'The page you tried to reach, restricts direct access.';
            $this->load->view('errors/html/error_404', $pagedata);
        }
        $waybill_data = ['$waybill_data' => ''];
        $country = $this->shipping_settings_model->_get_country();

        if ($country->value === 'Nigeria') {
            $waybill_data['states'] = $this->shipping_settings_model->_get_nigeria() ?: [];
        } else {
            $waybill_data['states'] = $this->shipping_settings_model->_get_uk() ?: [];
        }

        $waybill_data['customers'] = $this->waybill_model->_get_customers() ?: [];
        $waybill_data['sales'] = $this->waybill_model->_get_temps() ?: [];

        $this->load->view('popups/admin/waybill', $waybill_data);
    }
}
