<?php
if (!defined('BASEPATH'))
  exit('No direct script access allowed');

class Payment_model extends CI_Model {

  function __construct() {
    parent::__construct();
  }

  function _sum_payment($client, $token='')
  {
    $this->db->select("SUM(amount) as total");
    $sum = $this->db->get_where('er_lodgement', ['customer'=>$client, 'token !='=>$token, 'status !='=>1]);
    if ( ! $sum->num_rows() > 0) return 0;
    return $sum->row('total');
  }

  function _sum_supply($client, $token='')
  {
    $this->db->select("SUM(total) as total");
    $sum = $this->db->get_where('er_sales', ['customer'=>$client, 'token !='=>$token, 'status !='=>1]);
    if ( ! $sum->num_rows() > 0) return 0;
    return $sum->row('total');
  }

  function _get($params) 
  {
    $this->db->where('status', 0);
    $payment = $this->db->get_where('er_payments', $params);
    if ( ! ($payment->num_rows() > 0)) return false;
    return $payment->row();
  }

  function _get_all($voucher) 
  {
    // if (!isset($_SESSION['prosc_admin'])) $this->db->where('warehouse', $_SESSION['prosc_warehouse']);
    $payments = $this->db->get_where('er_payments', ['status'=>0, 'supply_id'=>$voucher]);
    if ( ! ($payments->num_rows() > 0)) return false;
    return $payments->result_array();
  }

  function _get_supply($voucher) 
  {
    $sale = $this->db->get_where('er_sales', ['voucher'=>$voucher, 'status'=>0]);
    if ( ! ($sale->num_rows() > 0)) return false;
    return $sale->result_array();
  }

  function _get_items($data = []) 
  {
    $items = $this->db->get('er_items');
    if ( ! ($items->num_rows() > 0)) return false;
    foreach ($items->result_array() as $row)
      $data[$row['token']] = ($row['status'] == 0) ? $row['name'] : ('**'.$row['name']);

    return $data;
  }

  function _get_sum($table, $col, $filter='')
  {
    $table = 'er_'.$table;
    $this->db->select('SUM('.$col.') as total');
    return $this->db->get_where($table, $filter)->row('total') ?? 0;
  }

  function _get_balances($target)
  {
    $payment = $this->_get_sum('payments', 'amount', ['supply_id'=>$target, 'status'=>0]);
    $supply = $this->_get_sum('sales', 'total', ['voucher'=>$target, 'status'=>0]);

    return ['supply'=>$supply, 'payment'=>$payment];
  }

  function _count()
  {
    return $this->db->count_all_results('er_payments');
  }

  function _put($params, $target) 
  {
    $this->db->where('token', $target);
    $user = $this->db->update('er_payments', $params);
    if ( ! $user) return false;
    return true;
  }

  function _post($data, $customer='')
  {
    $hash = base64_encode(random_bytes(32).date('Hisa'));
    $hash = strtr($hash, '+/', '-_');
    $hash = rtrim($hash, '=');
    $last = $this->_count() + 800001;
    $sale = $this->_get_supply($data['supply_id']);
    if ( ! $sale) return false;
    if ( ! array_key_exists('pid', $data))
      $data['pid']    = 'PID-'.$last;
    $data['token']    = $hash;
    $data['done_by']  = $this->session->userdata('prosc_user');
    $data['payer_id'] = $customer ?: $sale[0]['customer'];
    if ( ! $this->db->insert('er_payments', $data)) return false;
    return $data['token'];
  }

  function _drop($target)
  {
    if ( ! is_array($target)) $this->db->where('token', $target);
    else $this->db->where($target);
    $this->db->update('er_payments', ['status'=>'1']);
  }
}
