<?php
if (!defined('BASEPATH'))
  exit('No direct script access allowed');

class Sales_chart_model extends CI_Model
{

  function __construct()
  {
    parent::__construct();
  }

  function _get($params)
  {
    $this->db->where('status', 0);
    $user = $this->db->get_where('er_sales', $params);
    if (! ($user->num_rows() > 0)) return false;
    return $user->row();
  }

  function _evaluate($item, $outlet, $qty)
  {
    $data = $this->db->get_where('er_outlet_store', ['item' => $item, 'warehouse' => $outlet, 'status' => 0]);
    if ($data->num_rows() < 1) return false;
    $data = $data->row();
    $sub = $data->quantity - $qty;
    if ($sub < 0) return false;

    $this->db->where('token', $data->token);
    $this->db->update('er_outlet_store', ['quantity' => $sub]);
    return true;
  }

  function _get_keyed($token)
  {
    $this->db->where('status', 0);
    $sales = $this->db->get_where('er_sales', ['voucher' => $token]);
    if (! ($sales->num_rows() > 0)) return false;
    return true;
  }

  function _get_client($token)
  {
    $outlet = $this->db->get_where('er_customers', ['token' => $token]);
    if (! ($outlet->num_rows() > 0)) return false;
    return $outlet->row();
  }

  function _get_all($params = '')
  {
    $where = [];
    if (isset($_SESSION['prosc_sales_chart_x'])) {
      $y = $_SESSION['prosc_sales_chart_y'] ?? date('Y-m-d');
      $where = ['er_sales.date >=' => $_SESSION['prosc_sales_chart_x'], 'er_sales.date <=' => $y];
    } else {
      $dated = isset($_SESSION['prosc_sales_chart_date']) ? $_SESSION['prosc_sales_chart_date'] : date('Y-m-d');
      $this->db->like('er_sales.date', $dated);
    }

    $this->db->select('*, SUM(total) as sales, SUM(total_cost_price) as costs, SUM(quantity) as quantity')
      ->group_by('item')->where($where)->order_by('voucher DESC');
    $users = $this->db->get_where('er_sales', ['status' => 0,  'warehouse' => $params]);
    if (! ($users->num_rows() > 0)) return [];
    return $users->result_array();
  }

  function _sales($params = '')
  {
    if (isset($_SESSION['prosc_sales_chart_x'])) {
      $y = $_SESSION['prosc_sales_chart_y'] ?? date('Y-m-d');
      $this->db->where('date >=', $_SESSION['prosc_sales_chart_x']);
      $this->db->where('date <=', $y);
    } else {
      $dated = isset($_SESSION['prosc_sales_chart_date']) ? $_SESSION['prosc_sales_chart_date'] : date('Y-m-d');
      $this->db->like('date', $dated);
    }

    $this->db->order_by('voucher DESC');


    $users = $this->db->get_where('er_sales', ['status' => 0, 'warehouse' => $params]);
    if (! ($users->num_rows() > 0)) return false;
    return $users->result_array();
  }

  function get_item_name($ref = '')
  {

    if ($ref != null) {
      $x = $this->db->get_where('er_items', ['status' => 0, 'token' => $ref])->row();
      if ($x->status == '1')
        return '<span class="text-danger">' . $x->name . ' (Deleted)</span>';
      else
        return $x->name;
    }
  }

  function _get_chart($params = '', $level)
  {
    $where = [];
    if (isset($_SESSION['prosc_sales_chart_x'])) {
      $y = $_SESSION['prosc_sales_chart_y'] ?? date('Y-m-d');
      $where = ['er_sales.date >=' => $_SESSION['prosc_sales_chart_x'], 'er_sales.date <=' => $y];
    } else {
      $dated = isset($_SESSION['prosc_sales_chart_date']) ? $_SESSION['prosc_sales_chart_date'] : date('Y-m-d');
      $this->db->like('er_sales.date', $dated);
    }

    $sort = $level == 'lowest' ? 'ASC' : 'DESC';
    $this->db->select('*, SUM(quantity) as quantity, SUM(total) as total');
    $this->db->group_by('item')->order_by('quantity', $sort)->limit(5);
    $data = $this->db->get_where('er_sales', ['status' => 0, 'warehouse' => $params]);
    if (! ($data->num_rows() > 0)) return false;

    // $data = $users->result_array(); 
    $sales_chart = $sales = [];

    for ($i = 0; $i < 5; $i++) {
      $sales[$i] = ['item' => '', 'quantity' => 0, 'total' => 0, 'price' => 0];
    }

    $k = 0;
    foreach ($data->result_array() as $row) {
      $sales[$k] = [
        'item'     => $this->_get_name($row['item']),
        'quantity' => $row['quantity'],
        'total'    => $row['total'],
        'price'    => $row['total']
      ];
      $k++;
      // $products = $this->db->get_where('er_items', ['status'=>0, 'token' => $val['item']]);
      // $product = $products->row();

      // if( ! array_key_exists($product->name, $sales_chart) ){
      //   $sales_chart[$product->name]  = array(
      //     'quantity' => $val['quantity'],
      //     'item'    => $product->name,
      //     'price' => $val['price']
      //   );
      // }else{
      //   $sales_chart[$product->name]  = array(
      //     'quantity' =>  $sales_chart[$product->name]['quantity'] + $val['quantity'],
      //     'item'    => $product->name,
      //     'price' => $val['price']
      //   );
      // } 
    }

    // $vol = [];


    // $sales_chart = (array) $sales_chart;
    // foreach($sales_chart as $row){
    //   array_push($vol, $row['quantity']);
    // }

    // if($level == 'lowest')
    //   array_multisort($vol, SORT_ASC, $sales_chart);
    // if($level == 'highest')
    //   array_multisort($vol, SORT_DESC, $sales_chart);

    // return array_slice($sales_chart,0,4);
    return $sales;
  }

  function _get_name($item)
  {
    return $this->db->get_where('er_items', ['token' => $item])->row('name') ?? 'Nil';
  }

  function _get_totals($warehouse)
  {
    $where = [];
    if (isset($_SESSION['prosc_sales_chart_x'])) {
      $y = $_SESSION['prosc_sales_chart_y'] ?? date('Y-m-d');
      $where = ['er_sales.date >=' => $_SESSION['prosc_sales_chart_x'], 'er_sales.date <=' => $y];
    } else {
      $dated = isset($_SESSION['prosc_sales_chart_date']) ? $_SESSION['prosc_sales_chart_date'] : date('Y-m-d');
      $this->db->like('er_sales.date', $dated);
    }

    $this->db->select('SUM(total) as sales, SUM(total_cost_price) AS costs')->where($where);
    $sales = $this->db->get_where('er_sales', ['warehouse' => $warehouse, 'status' => 0]);
    if (! $sales->num_rows() > 0) return ['costs' => 0, 'sales' => 0];
    return $sales->row_array();
  }

  function _get_states()
  {
    $users = $this->db->get_where('er_nigeria_state', ['status' => 0]);
    if (! ($users->num_rows() > 0)) return false;
    return $users->result_array();
  }

  function _get_users($params = '')
  {
    if ($params != '') $this->db->where($params);
    $users = $this->db->get_where('er_admin', ['status' => 0]);
    if (! ($users->num_rows() > 0)) return false;
    return $users->result_array();
  }

  function _get_outlet($token)
  {
    $outlet = $this->db->get_where('er_warehouse', ['token' => $token, 'status' => 0]);
    $staff  = $this->db->get_where('er_admin', ['warehouse' => $token, 'status' => 0]);
    if (! ($outlet->num_rows() > 0) or ! ($staff->num_rows() > 0)) return false;
    $data['name']  = $outlet->row('name') ?? '<i>unavailable</i>';
    $data['staff'] = $staff->row('name') ?? '<i>unavailable</i>';
    $data['phone'] = $staff->row('phone') ?? '<i>unavailable</i>';
    return $data;
  }

  function _get_clients()
  {
    if (!isset($_SESSION['prosc_admin'])) $this->db->where('warehouse', $_SESSION['prosc_warehouse']);
    $this->db->order_by('serial ASC');
    $outlets = $this->db->get_where('er_customers', ['id >' => 1, 'status' => 0]);
    if (! ($outlets->num_rows() > 0)) return false;
    return $outlets->result_array();
  }
  function _get_sum($table, $col, $filter = '')
  {
    $this->db->select_sum($col);
    return $this->db->get_where($table, $filter)->row($col);
  }

  function _get_sums()
  {
    $data = [];
    $this->db->select("voucher, SUM(total) as total");
    $this->db->group_by('voucher');
    if (!isset($_SESSION['prosc_admin'])) $this->db->where('warehouse', $_SESSION['prosc_warehouse']);
    $sql = $this->db->get_where('er_sales', ['status' => 0, 'type' => 0]);
    if ($sql->num_rows() > 0) {
      $result = $sql->result_array();
      foreach ($result as $row) {
        $data[$row['voucher']] = $row['total'];
      }
    }
    return $data;
  }

  function _get_price($item, $state)
  {
    $items = $this->db->get_where('er_items', ['token' => $item]);
    if (! ($items->num_rows() > 0)) return 0;
    $price = json_decode($items->row('price'), true);
    return $price[$state];
  }

  function _get_items()
  {
    $this->db->order_by('name ASC');
    $items = $this->db->get_where('er_items', ['status' => 0]);
    if (! ($items->num_rows() > 0)) return false;
    $items = $items->result_array();
    $item  = [];
    foreach ($items as $row) {
      $k = json_decode($row['price'], true) ?? [];
      $k = $k['a577f519b58b533e400cf28a7464576babf92121'] ?? 0;
      $item[] = ['token' => $row['token'], 'name' => $row['name'], 'price' => number_format($k[0], 1), 'cost_price' => number_format($k[1], 1), 'category' => $row['category'], 'status' => $row['status']];
    }
    return $item;
  }

  function _get_cats()
  {
    $items = $this->db->get_where('er_categories', ['status' => 0]);
    if (! ($items->num_rows() > 0)) return false;
    return $items->result_array();
  }

  function _close()
  {
    $code = $_SESSION['prosc_sales_key'];
    $date = $_SESSION['prosc_sales_date'];
    $this->db->where(['voucher' => $code, 'date' => $date]);
    $this->db->update('er_sales', ['save' => '1']);
    return true;
  }

  function _put($params, $target)
  {
    if (is_array($target)) $this->db->where($target);
    else $this->db->where('token', $target);
    $user = $this->db->update('er_sales', $params);
    if (! $user) return false;
    return true;
  }

  function _count($filter = '')
  {
    if (is_array($filter)) $this->db->where($filter);
    else $this->db->group_by('voucher');
    return $this->db->count_all_results('er_sales');
  }

  function _post($data)
  {
    $hash = base64_encode(random_bytes(32) . date('Hisa'));
    $hash = strtr($hash, '+/', '-_');
    $hash = rtrim($hash, '=');
    $last = $this->_count() + 1;
    $uuid = str_repeat('0', 3 - strlen($last)) . $last;
    $data['token'] = $hash;
    if (! $this->db->insert('er_sales', $data)) return false;
    return $data['token'];
  }

  function _drop($target)
  {
    $this->db->where('token', $target);
    $this->db->update('er_sales', ['status' => '1']);
  }
}
