
<?php
if (!defined('BASEPATH'))
  exit('No direct script access allowed');

class Supply_client_model extends CI_Model
{

  function __construct()
  {
    parent::__construct();
  }

  function _get($params)
  {
    $this->db->where('status', 0);
    $user = $this->db->get_where('er_sales', $params);
    if (! ($user->num_rows() > 0)) return false;
    return $user->row();
  }

  function _get_keyed($token)
  {
    $this->db->where('status', 0);
    $sales = $this->db->get_where('er_sales', ['voucher' => $token]);
    if (! ($sales->num_rows() > 0)) return false;
    return true;
  }

  function _get_client($token)
  {
    $outlet = $this->db->get_where('er_customers', ['token' => $token]);
    if (! ($outlet->num_rows() > 0)) return false;
    return $outlet->row();
  }

  function _get_all($params = '')
  {
    if (!isset($_SESSION['prosc_admin'])) $this->db->where('warehouse', $_SESSION['prosc_warehouse']);
    if (isset($_SESSION['prosc_supply_client_key'])) $this->db->where('voucher', $_SESSION['prosc_supply_client_key']);
    if ($params != '') $this->db->where($params);
    $this->db->order_by('voucher DESC');
    if ($this->session->userdata('prosc_supply_client_key') == '' && ! isset($_SESSION['prosc_supply_clients_x']))
      $this->db->group_by('voucher')->limit('100');
    if (isset($_SESSION['prosc_supply_clients_x']))
      $this->db->where("`date` BETWEEN '" . $_SESSION['prosc_supply_clients_x'] . "' AND '" . $_SESSION['prosc_supply_clients_y'] . "'")->group_by('voucher');

    $users = $this->db->get_where('er_sales', ['status' => 0, 'type' => 1]);
    if (! ($users->num_rows() > 0)) return false;
    return $users->result_array();
  }

  function _get_states()
  {
    $users = $this->db->get_where('er_nigeria_state', ['status' => 0]);
    if (! ($users->num_rows() > 0)) return false;
    return $users->result_array();
  }

  function _get_clients()
  {
    if (! isset($_SESSION['prosc_admin'])) $this->db->where('warehouse', $_SESSION['prosc_warehouse']);
    $this->db->order_by('serial ASC');
    $outlets = $this->db->get_where('er_customers', ['status' => 0]);
    if (! ($outlets->num_rows() > 0)) return false;
    return $outlets->result_array();
  }

  function _get_sums()
  {
    $data = [];
    $this->db->select("voucher, SUM(total) as total");
    $this->db->group_by('voucher');
    if (! isset($_SESSION['prosc_admin'])) $this->db->where('warehouse', $_SESSION['prosc_warehouse']);
    $sql = $this->db->get_where('er_sales', ['status' => 0, 'type' => 1]);
    if ($sql->num_rows() > 0) {
      $result = $sql->result_array();
      foreach ($result as $row) {
        $data[$row['voucher']] = $row['total'];
      }
    }
    return $data;
  }

  function _get_price($item, $state)
  {
    $items = $this->db->get_where('er_items', ['token' => $item]);
    if (! ($items->num_rows() > 0)) return 0;
    $price = json_decode($items->row('price'), true);
    return $price[$state];
  }

  function _get_items()
  {
    $this->db->order_by('name ASC');
    $items = $this->db->get_where('er_items', ['status' => 0]);
    if (! ($items->num_rows() > 0)) return false;
    $items = $items->result_array();
    $item  = [];
    foreach ($items as $row) {
      $k = json_decode($row['price'], true) ?? [];
      $k = $k['a577f519b58b533e400cf28a7464576babf92121'] ?? [0, 0];
      $item[] = ['token' => $row['token'], 'name' => $row['name'], 'price' => number_format($k[0], 1), 'cost_price' => number_format($k[1], 1), 'category' => $row['category'], 'status' => $row['status']];
    }
    return $item;
  }

  function _get_cats()
  {
    $items = $this->db->get_where('er_categories', ['status' => 0]);
    if (! ($items->num_rows() > 0)) return false;
    return $items->result_array();
  }

  function _close()
  {
    $code = $_SESSION['prosc_supply_client_key'];
    $date = $_SESSION['prosc_supply_client_date'];
    $this->db->where(['voucher' => $code, 'date' => $date]);
    $this->db->update('er_sales', ['save' => '1']);
    return true;
  }

  function _put($params, $target)
  {
    if (is_array($target)) $this->db->where($target);
    else $this->db->where('token', $target);
    $user = $this->db->update('er_sales', $params);
    if (! $user) return false;
    return true;
  }

  function _count($filter = '')
  {
    if (is_array($filter)) $this->db->where($filter);
    else $this->db->group_by('voucher');
    return $this->db->count_all_results('er_sales');
  }

  function _post($data)
  {
    $hash = base64_encode(random_bytes(32) . date('Hisa'));
    $hash = strtr($hash, '+/', '-_');
    $hash = rtrim($hash, '=');
    $last = $this->_count() + 1;
    $uuid = str_repeat('0', 3 - strlen($last)) . $last;
    $data['token'] = $hash;
    $data['type']  = 1;
    if (! $this->db->insert('er_sales', $data)) return false;
    return $data['token'];
  }

  function _drop($target)
  {
    $this->db->where('token', $target);
    $this->db->update('er_sales', ['status' => '1']);
  }
}
