<?php defined('BASEPATH') or exit('No direct script access allowed'); ?>
<!DOCTYPE html>
<html lang="en">
<meta charset="utf-8">
<link rel="icon" type="image" href="<?= base_url('assets/images/favicon.png'); ?>" />
<link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-9ndCyUaIbzAi2FUVXJi0CjmCapSmO7SnpJef0486qhLnuZ2cdeRhO02iuK6FUUVM" crossorigin="anonymous">
<link rel="stylesheet" href="<?= base_url('assets/css/receipt.css'); ?>" />
<script src="https://code.jquery.com/jquery-3.7.0.min.js" integrity="sha256-2Pmvv0kuTBOenSvLm6bvfBSSHrUJ+3A7x6P5Ebd07/g=" crossorigin="anonymous"></script>
<title>Xcel POS - Customer View</title>
</head>

<body>
    <section class="w-100 h-100 bg-light" style="display: flex; justify-content:center; align-items: center;">
        <div class="d-flex gap-2 flex-column p-2" style="width: 95%; height: 90%; background: white; border-radius: 20px">
            <div class="d-flex gap-5 w-100 p-2" style="box-shadow: 5px 5px 10px rgba(0, 0, 0, 0.2); border-radius: 10px;">
                <div class=" d-flex gap-1 flex-column">
                    <p class="fw-bold">Date</p>
                    <span id="customer-date"></span>
                </div>
                <div class="d-flex gap-1 flex-column">
                    <p class="fw-bold">Time</p>
                    <span id="customer-time"></span>
                </div>
            </div>

            <div class="d-flex flex-column w-100 p-2" id="customer-items" style="overflow-y: auto; height: 85%;">

                <p class="text-muted text-center">No items selected...</p>

            </div>

            <div class="d-flex flex-column gap-1 w-100 p-2" style="height: 15%; border-top: 1px dotted black; display: flex; align-items: flex-end;">
                <div class=" d-flex gap-3 flex-row">
                    <p class=" fw-bold">Sub total :</p>
                    <?php if ($currency == "GBP"): ?>
                        <span>£ </span>
                        <span id="customer-subtotal">0</span>
                    <?php else: ?>
                        <span class="money"></span>
                        <p id="customer-subtotal">0</p>
                    <?php endif; ?>

                </div>
                <div class="d-flex gap-2">
                    <p class=" fw-bold">Tax :</p>
                    <?php if ($currency == "GBP"): ?>
                        <span>£ </span>
                        <p id="customer-tax">0</p>
                    <?php else: ?>
                        <span class="money">
                        </span>
                        <p id="customer-tax">0</p>
                    <?php endif; ?>
                </div>
                <div class="d-flex gap-2">
                    <p class=" fw-bold">Total :</p>
                    <?php if ($currency == "GBP"): ?>
                        <span>£ </span>
                        <p id="customer-total">0</p>
                    <?php else: ?>
                        <span class="money">
                        </span>
                        <p id="customer-total">0</p>
                    <?php endif; ?>
                </div>
            </div>
        </div>
    </section>
    <script>
        const dateOptions = {
            weekday: 'short',
            day: '2-digit',
            month: 'short',
            year: 'numeric'
        };
        const timeOptions = {
            hour: '2-digit',
            minute: '2-digit',
            second: '2-digit'
        };
        const now = new Date();
        document.getElementById('customer-date').textContent = now.toLocaleDateString('en-US', dateOptions);
        document.getElementById('customer-time').textContent = now.toLocaleTimeString('en-US', timeOptions);

        let selectedItems = JSON.parse(localStorage.getItem('selectedItems')) || [];
        const TAX_RATE = 0.05; // Example tax rate: 5%
        function renderCustomerItems() {
            const itemsContainer = document.getElementById('customer-items');
            itemsContainer.innerHTML = ''; // Clear previous content

            if (selectedItems.length === 0) {
                itemsContainer.innerHTML = '<p class="text-muted text-center">No items selected...</p>';
                updateCustomerTotals();
                return;
            }

            selectedItems.forEach(item => {
                const itemHTML = `
                    <div class="d-flex gap-2" style="width: 100%; height: 25%; padding: 15px 0px; border-bottom: 1px dotted black;">
                      <img
                        style="width:100px; height: 100px; object-fit: cover; border-radius: 5px;"
                        src="${item.image}"
                        alt="${item.name}">

                        <div class="" style="width: 100%; height: 80%;">
                            <p class="fw-bold">${item.name}</p>
                            <p class="p-1 bg-info text-dark rounded-3" style="width: 150px;">${item.category}</p>
                            <div class="d-flex justify-content-between w-100 p-2">
                                <p class="mb-0">${item.quantity} x 
                                <?php if ($currency == "GBP"): ?>
                                    <span>£ </span>
                                    ${item.price.toLocaleString()}
                                <?php else: ?>
                                    <span class="money">
                                    </span>
                                        ${item.price.toLocaleString()}
                                <?php endif; ?>
                                </p>
                                <p class="fw-bold mb-0">
                                 <?php if ($currency == "GBP"): ?>
                                    <span>£ </span>
                                    ${(item.quantity * item.price).toLocaleString()}
                                <?php else: ?>
                                    <span class="money">
                                    </span>
                                    ${(item.quantity * item.price).toLocaleString()}
                                <?php endif; ?>
                               
                                </p>
                           </div>
                       </div>`;
                itemsContainer.innerHTML += itemHTML;
            });

            updateCustomerTotals();
        }

        function updateCustomerTotals() {
            const subtotal = selectedItems.reduce((sum, item) => sum + item.price * item.quantity, 0);
            const tax = subtotal * TAX_RATE;
            const total = subtotal + tax;

            document.getElementById('customer-subtotal').textContent = subtotal.toLocaleString();
            document.getElementById('customer-tax').textContent = tax.toLocaleString();
            document.getElementById('customer-total').textContent = total.toLocaleString();
        }

        // Load items on page load
        window.onload = function() {
            selectedItems = JSON.parse(localStorage.getItem('selectedItems')) || [];
            renderCustomerItems();
        };

        // Listen for changes in the selected items and re-render
        window.addEventListener('storage', function(event) {
            if (event.key === 'selectedItems') {
                selectedItems = JSON.parse(event.newValue) || [];
                renderCustomerItems();
            }
        });
    </script>
</body>