<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class Account_control extends CI_Controller {
  
  public function __construct()
  {
    parent::__construct();

    if ($this->session->userdata('prosc_admin') !== true)
      redirect(base_url(), 'refresh');

    $this->load->model('account_model');
  }

  public function index()
  {
    $this->pager();
  }

  function pager($tmp='')
  {
    if (!$this->input->is_ajax_request() && strstr(current_url(), '.aspx') == '') redirect(base_url('not_found.aspx'), 'refresh');
    $this->db->order_by('name');
    $data = $this->account_model->_get_all() ?: [];
    $datac = $this->account_model->_get_account_types() ?: [];
    if ($tmp == '') $this->load->view('default', ['page'=>'chart_of_account', 'data'=>$data, 'tmp'=>$tmp, 'account_types' => $datac]);
    else return $this->load->view('admin/chart_of_account', ['data'=>$data, 'tmp'=>$tmp, 'account_types' => $datac], true);
  }

  function check($token='')
  {
    $name  = trim($this->input->post('name'));
    $description  = trim($this->input->post('description'));
    $account_type = trim($this->input->post('account_type'));
   
    if ($name == '' OR $account_type == '') return false;
    if ( ! ctype_alpha(str_replace(' ', '', $name))) return 'Account name is invalid';
    if ($description != '' && ! ctype_alpha(str_replace(' ', '', $description))) return 'Description is invalid';
    if (is_numeric($description)) return 'Description cannot be a number';
    if ($this->account_model->_get(['name'=>$name, 'token !='=>$token])) return 'Account: <b>'. strtoupper($name) .'</b> already exists';

    return [
      'name'          => $name,
      'description'   => $description,
      'expenses_type' => $account_type,
      'done_by'       => $this->session->userdata('prosc_user')
    ];
  }

  function create()
  {
    $data = $this->check();
    if ( ! $data) exit(json_encode(['status'=>'error', 'message'=>'Fill in all fields']));
    if ( ! is_array($data)) exit(json_encode(['status'=>'error', 'message'=>$data]));
    
    if ( ! $this->account_model->_post($data))
      exit(json_encode(['status'=>'error', 'message'=>'An unknown error occured and the insertion operation failed']));

    echo (json_encode(['status'=>'success', 'message'=>$this->pager('dump')]));
  }

  function update()
  {
    $target = trim($this->input->get('target'));
    if ( ! $this->account_model->_get(['token'=>$target])) exit(json_encode(['status'=>'error', 'message'=>'A fatal error has occured. <br />Please refresh and try again']));
    $data = $this->check($target);
    if ( ! $data) exit(json_encode(['status'=>'error', 'message'=>'Fill in all fields']));
    if ( ! is_array($data)) exit(json_encode(['status'=>'error', 'message'=>$data]));

    if ( ! $this->account_model->_put($data, $target))
      exit(json_encode(['status'=>'error', 'message'=>'An unknown error occured and the update failed']));
    
    echo (json_encode(['status'=>'success', 'message'=>$this->pager('dump')]));
  }

  function delete()
  {
    $target = trim($this->input->get('token'));
    if ( ! $this->account_model->_get(['token'=>$target])) exit(json_encode(['status'=>'error', 'message'=>'A fatal error has occured. <br />Please refresh and try again']));

    $this->account_model->_drop($target); 
    $this->session->set_flashdata('prosc_message', 'Record Deleted');
    echo (json_encode(['status'=>'success', 'message'=>$this->pager('dump')]));
  }

  function popup()
  {
    if ( ! $this->input->is_ajax_request()) {
      $pagedata['heading'] = 'Hold on!';
      $pagedata['message'] = 'The page you tried to reach, restricts direct access.';
      $this->load->view('errors/html/error_404', $pagedata);
    }
    $last = $this->account_model->_count() + 1;
    $target = $this->input->get('token');
    $data = ['data'=>'', 'last'=>$last];
    $data['account_types'] = $this->account_model->_get_account_types() ?: [];
    $file = $this->account_model->_get(['token'=>$target]);
    if ($file) $data['data'] = $file;
    $this->load->view('popups/admin/account', $data);
  }
}
