<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Account_type_control extends CI_Controller
{

  public function __construct()
  {
    parent::__construct();

    if ($this->session->userdata('prosc_admin') !== true)
      redirect(base_url(), 'refresh');

    $this->load->model('account_type_model');
  }

  public function index()
  {
    $this->pager();
  }

  function pager($tmp = '')
  {
    if (!$this->input->is_ajax_request() && strstr(current_url(), '.aspx') == '') redirect(base_url('not_found.aspx'), 'refresh');
    $this->db->order_by('name');
    $data = $this->account_type_model->_get_all() ?: [];
    if ($tmp == '') $this->load->view('default', ['page' => 'account_type', 'data' => $data, 'tmp' => $tmp,]);
    else return $this->load->view('admin/account_type', ['data' => $data, 'tmp' => $tmp], true);
  }

  function check($condition)
  {
    if ($condition == 'create') {
      $name  = trim($this->input->post('name'));
      $type  = trim($this->input->post('type'));
      $descr = trim($this->input->post('description'));

      if ($name == '' or !in_array($type, [0, 1])) return false;
      if (!ctype_alpha(str_replace(' ', '', $name))) return 'Account type name is invalid';
      // if ($descr != '' && ! ctype_alpha(str_replace(' ', '', $descr))) return 'Description is invalid';
      // if (is_numeric($descr)) return 'Description cannot be a number';
      if ($this->account_type_model->_get(['name' => $name])) return 'Account type: <b>' . strtoupper($name) . '</b> already exists';

      return [
        'name'        => $name,
        'type'        => $type,
        'description' => $descr,
        'done_by'     => $this->session->userdata('prosc_user')
      ];
    } else {
      $name  = trim($this->input->post('name'));
      $type  = trim($this->input->post('type'));
      $descr = trim($this->input->post('description'));

      if ($name == '' or !in_array($type, [0, 1])) return false;
      if (!ctype_alpha(str_replace(' ', '', $name))) return 'Account type name is invalid';
      // if ($descr != '' && ! ctype_alpha(str_replace(' ', '', $descr))) return 'Description is invalid';
      // if (is_numeric($descr)) return 'Description cannot be a number';
      return [
        'name'        => $name,
        'type'        => $type,
        'description' => $descr,
        'done_by'     => $this->session->userdata('prosc_user')
      ];
    }
  }

  function create()
  {
    $data = $this->check('create');
    if (!$data) exit(json_encode(['status' => 'error', 'message' => 'Fill in required fields']));
    if (!is_array($data)) exit(json_encode(['status' => 'error', 'message' => $data]));

    if (!$this->account_type_model->_post($data))
      exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured and the insertion operation failed']));

    echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
  }

  function update()
  {
    $target = trim($this->input->get('target'));
    if (!$this->account_type_model->_get(['token' => $target])) exit(json_encode(['status' => 'error', 'message' => 'A fatal error has occured. <br />Please refresh and try again']));
    $data = $this->check('update');
    if (!$data) exit(json_encode(['status' => 'error', 'message' => 'Fill in required fields']));
    if (!is_array($data)) exit(json_encode(['status' => 'error', 'message' => $data]));

    if (!$this->account_type_model->_put($data, $target))
      exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured and the update failed']));

    echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
  }

  function delete()
  {
    $target = trim($this->input->get('token'));
    if (!$this->account_type_model->_get(['token' => $target])) exit(json_encode(['status' => 'error', 'message' => 'A fatal error has occured. <br />Please refresh and try again']));

    $this->account_type_model->_drop($target);
    $this->session->set_flashdata('prosc_message', 'Record Deleted');
    echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
  }

  function popup()
  {
    if (!$this->input->is_ajax_request()) {
      $pagedata['heading'] = 'Hold on!';
      $pagedata['message'] = 'The page you tried to reach, restricts direct access.';
      $this->load->view('errors/html/error_404', $pagedata);
    }
    $last = $this->account_type_model->_count() + 1;
    $target = $this->input->get('token');
    $data = ['data' => '', 'last' => $last];
    $file = $this->account_type_model->_get(['token' => $target]);
    if ($file) $data['data'] = $file;
    $this->load->view('popups/admin/account_type', $data);
  }
}
