<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Inventory_category_control extends CI_Controller
{

  public function __construct()
  {
    parent::__construct();

    if ($this->session->userdata('prosc_admin') !== true)
      redirect(base_url(), 'refresh');

    $this->load->model('inventory_category_model');
  }

  public function index()
  {
    $this->pager();
  }

  function pager($tmp = '')
  {
    if (!$this->input->is_ajax_request() && strstr(current_url(), '.aspx') == '') redirect(base_url('not_found.aspx'), 'refresh');
    $this->db->order_by('created_at DESC');
    $data = $this->inventory_category_model->_get_all() ?: [];
    if ($tmp == '') $this->load->view('default', ['page' => 'inventory_category', 'data' => $data, 'tmp' => $tmp]);
    else return $this->load->view('admin/inventory_category', ['data' => $data, 'tmp' => $tmp], true);
  }

  function check()
  {
    $name = trim($this->input->post('name'));
    $par  = trim($this->input->post('parent'));
    $descr = trim($this->input->post('description'));
    if ($name == '') return false;
    if (!ctype_alnum(str_replace(' ', '', $name))) return 'Category name cannot contain special characters';
    if (is_numeric($name)) return 'Category name cannot be a number';
    if ($par != '' && !$this->inventory_category_model->_get(['token' => $par])) return 'Parent Category Error!';

    return [
      'name'        => $name,
      'parent'      => $par,
      'description' => $descr
    ];
  }

  function create()
  {
    $data = $this->check();
    if (!$data) exit(json_encode(['status' => 'error', 'message' => 'Enter Category name']));
    if (!is_array($data)) exit(json_encode(['status' => 'error', 'message' => $data]));

    $k = $data;
    unset($k['color'], $k['description']);
    if ($this->inventory_category_model->_get($k))
      exit(json_encode(['status' => 'error', 'message' => "Category: <b>" . $data['name'] . "</b> already exists"]));

    if (!$this->inventory_category_model->_post($data))
      exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured and the insertion operation failed']));

    echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
  }

  function update()
  {
    $target = trim($this->input->get('target'));
    if (!$this->inventory_category_model->_get(['token' => $target])) exit(json_encode(['status' => 'error', 'message' => 'A fatal error has occured. <br />Please refresh and try again']));
    $data = $this->check();
    if (!$data) exit(json_encode(['status' => 'error', 'message' => 'Enter Category name']));
    if (!is_array($data)) exit(json_encode(['status' => 'error', 'message' => $data]));
    
    $k = $data;
    unset($k['color']);
    $dbData = $this->inventory_category_model->_get($k);
    if ($this->inventory_category_model->_get($k))
      exit(json_encode(['status' => 'error', 'message' => "Category: <b>" . $data['name'] . "</b> already exists"]));

    if ($dbData['description'] == $k['description'] && $dbData['name'] ==  $k['name'])
      exit(json_encode(['status' => 'error', 'message' => "Category: <b>" . $data['name'] . "</b> already exists"]));

    if (!$this->inventory_category_model->_put($data, $target))
      exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured and the update failed']));

    echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
  }

  function delete()
  {
    $target = trim($this->input->get('token'));
    if (!$this->inventory_category_model->_get(['token' => $target]))
      exit(json_encode(['status' => 'error', 'message' => 'A fatal error has occured. <br />Please refresh and try again']));

    $this->inventory_category_model->_drop($target);
    echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
  }

  function popup()
  {
    if (!$this->input->is_ajax_request()) {
      $pagedata['heading'] = 'Hold on!';
      $pagedata['message'] = 'The page you tried to reach, restricts direct access.';
      $this->load->view('errors/html/error_404', $pagedata);
    }
    $target = $this->input->get('token');
    $this->db->order_by('name');
    $data = ['data' => '', 'cats' => $this->inventory_category_model->_get_all(['parent' => '', 'token !=' => $target])];
    $file = $this->inventory_category_model->_get(['token' => $target]);
    if ($file) $data['data'] = $file;
    $this->load->view('popups/admin/category', $data);
  }
}
