<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Items_control extends CI_Controller
{

  public function __construct()
  {
    parent::__construct();

    if ($this->session->userdata('prosc_admin') !== true)
      redirect(base_url(), 'refresh');

    $this->load->model('items_model');
  }

  public function index()
  {
    $this->pager();
  }

  function stock()
  {
    $value = trim($this->input->post('value'));

    if (!is_numeric($value) or $value < 1)  exit(json_encode(['status' => 'error', 'message' => 'Minimum Invertory Stock cannot be less than 1']));
    $this->items_model->_set_min($value);
    $this->session->set_flashdata('prosc_message', "Minimum Invertory Stock is now set to $value");
    echo (json_encode(['status' => 'success', 'message' => '', 'extra' => '']));
  }

  function pager($tmp = '')
  {
    if (!$this->input->is_ajax_request() && strstr(current_url(), '.aspx') == '') redirect(base_url('not_found.aspx'), 'refresh');
    $data = $this->items_model->_get_all() ?: [];
    $datac = $this->items_model->_get_categories() ?: [];
    $states = $this->items_model->_get_states();
    $s = [];
    $stock = $this->notification_model->_get_min() ?: 1;
    if ($tmp == '') $this->load->view('default', ['page' => 'items', 'data' => $data, 'categories' => $datac, 'states' => $states, 'tmp' => $tmp, 'stock' => $stock]);
    else return $this->load->view('admin/items', ['data' => $data, 'categories' => $datac, 'states' => $states, 'tmp' => $tmp], true);
  }

  function check()
  {
    $name  = trim($this->input->post('name'));
    $unit  = trim($this->input->post('unit'));
    $price = intval(trim($this->input->post('price')));
    $cost_price = intval(trim($this->input->post('cost_price')));
    $cate  = trim($this->input->post('category'));
    $descr = trim($this->input->post('description'));
    $image = $this->input->post('image');
    $min_stock = $this->input->post('min_stock');
    $discount = $this->input->post('discount');
    $vat = 0;
    $stock = $this->input->post('stock');
    if ($name == '' or $price == '' or $cost_price == '' or $cate == '' or $unit == '' or $min_stock == '' or $vat == '' or $stock == '') return false;
    // if ( ! ctype_alnum(str_replace(' ', '', $name))) return 'Product name cannot contain special characters';
    if (is_numeric($name)) return 'Product name cannot be a number';
    if (!is_numeric($price) or $price < 1) return 'Product price must be a number greater than zero';
    if (!is_numeric($cost_price) or $cost_price < 1) return 'Product Cost price must be a number greater than zero';

    $data = [
      'name'        => $name,
      'description' => $descr,
      'unit'        => $unit,
      'price'       => json_encode(['a577f519b58b533e400cf28a7464576babf92121' => [$price, $cost_price]]),
      'category'    => $cate,
      'discount'    => $discount,
      'vat'    => $vat,
      'stock'    => $stock,
      'min_stock'    => $min_stock,
      'done_by'     => $this->session->userdata('prosc_admin')
    ];
    if ($image) $data['image'] = base64_decode($image);
    return $data;
  }

  function create()
  {
    $data = $this->check();
    if (!$data) exit(json_encode(['status' => 'error', 'message' => 'Fill in all required fields']));
    if (!is_array($data)) exit(json_encode(['status' => 'error', 'message' => $data]));

    if ($this->items_model->_get(['name' => $data['name'], 'unit' => $data['unit'], 'category' => $data['category']]))
      exit(json_encode(['status' => 'error', 'message' => 'A similar item already exists in inventory']));
    if (!$this->items_model->_post($data))
      exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured and the insertion operation failed']));

    echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
  }

  function update()
  {
    $target = trim($this->input->get('target'));
    if (!$this->items_model->_get(['token' => $target])) exit(json_encode(['status' => 'error', 'message' => 'A fatal error has occured. <br />Please refresh and try again']));
    $data = $this->check();
    if (!$data) exit(json_encode(['status' => 'error', 'message' => 'Fill in all required fields']));
    if (!is_array($data)) exit(json_encode(['status' => 'error', 'message' => $data]));

    if ($this->items_model->_get(['token !=' => $target, 'name' => $data['name'], 'unit' => $data['unit'], 'category' => $data['category']]))
      exit(json_encode(['status' => 'error', 'message' => 'A similar item already exists in inventory']));

    if (!$this->items_model->_put($data, $target))
      exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured and the update failed']));

    echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
  }

  function delete()
  {
    $target = trim($this->input->get('token'));
    if (!$this->items_model->_get(['token' => $target])) exit(json_encode(['status' => 'error', 'message' => 'A fatal error has occured. <br />Please refresh and try again']));

    $this->items_model->_drop($target);
    echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
  }

  function popup()
  {
    if (!$this->input->is_ajax_request()) {
      $pagedata['heading'] = 'Hold on!';
      $pagedata['message'] = 'The page you tried to reach, restricts direct access.';
      $this->load->view('errors/html/error_404', $pagedata);
    }
    $last = $this->items_model->_count() + 1;
    $target = $this->input->get('token');
    $data = ['data' => '', 'last' => $last];
    $data['categories'] = $this->items_model->_get_categories() ?: [];
    $file = $this->items_model->_get(['token' => $target]);
    $states = $this->items_model->_get_states();
    $s = [];
    if ($file) $data['data'] = $file;
    $data['states'] = $states;
    $this->load->view('popups/admin/item', $data);
  }
}
