<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Kpi_control extends CI_Controller
{

    public function __construct()
    {
        parent::__construct();

        if ($this->session->userdata('prosc_admin') !== true)
            redirect(base_url(), 'refresh');

        $this->load->model('kpi_model');
        $this->load->model('shipping_settings_model');
    }

    public function index()
    {
        $this->pager();
    }


    function pager($tmp = '')
    {
        if (!$this->input->is_ajax_request() && strstr(current_url(), '.aspx') == '') redirect(base_url('not_found.aspx'), 'refresh');
        $data = $this->kpi_model->_get_all() ?: [];
        $stock = $this->notification_model->_get_min() ?: 1;
        if ($tmp == '') $this->load->view('default', ['page' => 'kpi', 'data' => $data, 'tmp' => $tmp, 'stock' => $stock]);
        else return $this->load->view('admin/kpi', ['data' => $data, 'tmp' => $tmp], true);
    }

    function check()
    {
        $name  = trim($this->input->post('name'));
        $description  = trim($this->input->post('description'));
        $outlet = trim($this->input->post('outlet'));
        if ($name == '' or $description == '' or $outlet == '' or $outlet == '...') return false;

        $data = [
            'name'        => $name,
            'description' => $description,
            'outlet'        => $outlet,
            'done_by'     => $this->session->userdata('prosc_admin'),
            'status'     => 0
        ];
        return $data;
    }

    function create()
    {
        $data = $this->check();
        if (!$data) exit(json_encode(['status' => 'error', 'message' => 'Fill in all required fields']));
        if (!is_array($data)) exit(json_encode(['status' => 'error', 'message' => $data]));

        if ($this->kpi_model->_get(['name' => $data['name']]))
            exit(json_encode(['status' => 'error', 'message' => 'A similar KPI already exists in KPI list']));
        if ($this->kpi_model->_get_staff_per($data['name']))
            exit(json_encode(['status' => 'error', 'message' => 'A similar KPI already exists in staff performance']));
        if (!$this->kpi_model->_post($data))
            exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured and the insertion operation failed']));

        echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
    }

    function update()
    {
        $target = trim($this->input->get('target'));
        if (!$this->kpi_model->_get(['token' => $target])) exit(json_encode(['status' => 'error', 'message' => 'A fatal error has occured. <br />Please refresh and try again']));
        $data = $this->check();
        if (!$data) exit(json_encode(['status' => 'error', 'message' => 'Fill in all required fields']));
        if (!is_array($data)) exit(json_encode(['status' => 'error', 'message' => $data]));

        if ($this->kpi_model->_get(['token !=' => $target, 'name' => $data['name'], 'unit' => $data['unit'], 'category' => $data['category']]))
            exit(json_encode(['status' => 'error', 'message' => 'A similar item already exists in inventory']));

        if (!$this->kpi_model->_put($data, $target))
            exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured and the update failed']));

        echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
    }

    function delete()
    {
        $target = trim($this->input->get('token'));
        if (!$this->kpi_model->_get(['token' => $target])) exit(json_encode(['status' => 'error', 'message' => 'A fatal error has occured. <br />Please refresh and try again']));

        $this->kpi_model->_drop($target);
        echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
    }

    function popup()
    {
        if (!$this->input->is_ajax_request()) {
            $pagedata['heading'] = 'Hold on!';
            $pagedata['message'] = 'The page you tried to reach, restricts direct access.';
            $this->load->view('errors/html/error_404', $pagedata);
        }

        $data['data'] = [];
        $file = $this->kpi_model->_get_outlets();
        $country = $this->shipping_settings_model->_get_country();

        if ($country->value === 'Nigeria') {
            $states = $this->shipping_settings_model->_get_nigeria() ?: [];
        } else {
            $states = $this->shipping_settings_model->_get_uk() ?: [];
        }
        $s = [];
        if ($file) $data['data'] = $file;
        $data['states'] = $states;
        $this->load->view('popups/admin/kpi', $data);
    }
}
