<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class Payment_control extends CI_Controller {
  
  public function __construct()
  {
    parent::__construct();

    if ($this->session->userdata('prosc_user') == '')
      redirect(base_url(), 'refresh');
    $this->load->model('payment_model');
  }

  public function index()
  {
    $this->pager();
  }

  function pager($tmp='')
  {
    if ( ! $this->input->is_ajax_request() && strstr(current_url(), '.aspx') == '') redirect(base_url('not_found.aspx'), 'refresh');
    $data = $this->payment_model->_get_all($this->input->get('voucher')) ?: [];
    $items = $this->payment_model->_get_items() ?: [];
    $supply = $this->payment_model->_get_supply($this->input->get('voucher')) ?: [];
    $balances = $this->payment_model->_get_balances($this->input->get('voucher'));
    $this->load->view('default', ['page'=>'payments', 'data'=>$data, 'supply'=>$supply, 'items'=>$items, 'balances'=>$balances]);
  }

  function check()
  {
    $supply = trim($this->input->post('supply'));
    $amount = trim(str_replace(',', '', $this->input->post('amount')));
    $amount = str_replace(' ', '', $amount);
    $dated = trim($this->input->post('dated'));
    if ($dated == '' OR $amount == '') return false;
    if ($supply == '') return 'An unknown error occured';
    if ($dated > date('Y-m-d')) return 'Invalid Date Supplied';
    if ( ! $this->payment_model->_get_supply($supply)) return 'Supply Voucher is invalid';
    if ( ! is_numeric($amount) OR $amount <= 0) return 'Payment amount is invalid';
    if ($this->input->get('target') != '') $this->db->where('token !=', $this->input->get('target'));
    $payment = $this->payment_model->_get_balances($supply);
    var_dump($supply);
    $total = $payment['supply'];
    $payment = $payment['payment'];
    if (($payment + $amount - $total) > 0) 
      return 'New Payment: <b>N'. number_format($amount) .'</b> + Current Balance: <b>N'. number_format($payment) .'</b><br /> exceeds Supply Total of: <b>N'. number_format($total) .'</b>';

    return [
      'supply_id' => $supply,
      'amount'    => $amount,
      'date'      => $dated,
      'paid'      => 1
    ];
  }

  function create()
  {
    $data = $this->check();
    if ( ! $data) exit(json_encode(['status'=>'error', 'message'=>'Some fields were not properly filled']));
    if ( ! is_array($data)) exit(json_encode(['status'=>'error', 'message'=>$data]));
    if ( ! $this->payment_model->_post($data))
      exit(json_encode(['status'=>'error', 'message'=>'An unknown error occured and the insertion operation failed']));

    // echo (json_encode(['status'=>'success', 'message'=>'']));
    echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));

  }

  function update()
  {
    $target = trim($this->input->get('target'));
    if ( ! $this->payment_model->_get(['token'=>$target])) exit(json_encode(['status'=>'error', 'message'=>'A fatal error has occured. <br />Please refresh and try again']));
    $data = $this->check();
    if ( ! $data) exit(json_encode(['status'=>'error', 'message'=>'Fill in all fields']));
    if ( ! is_array($data)) exit(json_encode(['status'=>'error', 'message'=>$data]));
    if ( ! $this->payment_model->_put($data, $target))
      exit(json_encode(['status'=>'error', 'message'=>'An unknown error occured and the update failed']));

    // echo (json_encode(['status'=>'success', 'message'=>'', 'extra'=>'reload']));
      echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
  }

  function delete()
  {
    $target = trim($this->input->get('token'));
    if ( ! $this->payment_model->_get(['token'=>$target])) exit(json_encode(['status'=>'error', 'message'=>'A fatal error has occured. <br />Please refresh and try again']));

    $this->payment_model->_drop($target);
    echo (json_encode(['status'=>'success', 'message'=>'', 'extra'=>'']));
  }

  function history()
  {
    $target = $this->input->get('target');
    $data = $this->payment_model->_get_history($target) ?: [];
    $payment = $this->payment_model->_get_payments($target) ?: [];
    $client = $this->payment_model->_get_client($target);
    $balances = $this->payment_model->_get_balances($target);
    $this->load->view('default', ['page'=>'payment_history', 'data'=>$data, 'payment'=>$payment, 'client'=>$client, 'balances'=>$balances]);
  }

  function popup()
  {
    if ( ! $this->input->is_ajax_request()) {
      $pagedata['heading'] = 'Hold on!';
      $pagedata['message'] = 'The page you tried to reach, restricts direct access.';
      $this->load->view('errors/html/error_404', $pagedata);
    }
    $data = ['data'=>''];
    $data['supply'] = $this->input->get('supply');
    $target = $this->input->get('token');
    $file = $this->payment_model->_get(['token'=>$target]);
    $balance = $this->payment_model->_get_balances($data['supply']);
    $balance = $balance['supply'] - $balance['payment'];
    if ($file) $data['data'] = $file;
    if ($file) $balance = $balance + $file->amount;
    $data['balance'] = $balance;
    $data['ref'] = "PID-". ($this->payment_model->_count() + 800001);
    $this->load->view('popups/admin/payment', $data);
  }
}
