<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class Salex_control extends CI_Controller {
  
  public function __construct()
  {
    parent::__construct();

    if (!isset($_SESSION['prosc_user']))
      redirect(base_url(), 'refresh');

    $this->load->model('salex_model');
    $this->load->model('inventory_model');
  }

  public function index()
  {
    $this->pager();
  }

  function pager($tmp='')
  {
    if (!$this->input->is_ajax_request() && strstr(current_url(), '.aspx') == '') redirect(base_url('not_found.aspx'), 'refresh');
    $cats = $this->salex_model->_get_cats() ?: [];
    $items = $this->salex_model->_get_items() ?: [];
    $data  = $this->salex_model->_get_all() ?: [];
    $datax  = $this->salex_model->_get_allx() ?: [];
    $totals = $this->salex_model->_get_sums() ?: [];
    $states = $this->salex_model->_get_states() ?: [];
    $outlets = $this->salex_model->_get_outlets() ?: [];
    if ($tmp == '') $this->load->view('default', ['page'=>'salex', 'cats'=>$cats, 'data'=>$data, 'totals'=>$totals, 'states'=>$states, 'outlets'=>$outlets, 'items'=>$items, 'tmp'=>$tmp]);
    else return $this->load->view('admin/salex', ['data'=>$data, 'cats'=>$cats, 'totals'=>$totals, 'states'=>$states, 'outlets'=>$outlets, 'items'=>$items, 'tmp'=>$tmp], true);
  }

  function check_detail($token='', $dated='', $client='', $byname='', $bphone='')
  {
    $name   = $token ?: trim($this->input->post('autogen'));
    $dated  = $dated ?: trim($this->input->post('dated'));
    $client = $client ?: trim($this->input->post('customer'));
    $byname = $byname ?: trim($this->input->post('name'));
    $bphone = $bphone ?: trim($this->input->post('phone'));

    if ($name == '' OR $dated == '' OR $byname == '' OR $bphone == '') exit(json_encode(['status'=>'error', 'message'=>'Fill in all fields']));
    if ( ! ctype_alpha(str_replace(' ', '', $byname))) exit(json_encode(['status'=>'error', 'message'=>'Enter a valid name']));
    if ( ! is_numeric($bphone) OR strlen($bphone) != 11) exit(json_encode(['status'=>'error', 'message'=>'Phone number must be 11 digits']));
    if ($dated > date('Y-m-d')) exit(json_encode(['status'=>'error', 'message'=>'Invalid date supplied']));
    if ($token == '' && $this->salex_model->_get_keyed($name)) exit(json_encode(['status'=>'error', 'message'=>'Request Error']));

    if ($token != '') $name = $token;
    $_SESSION['prosc_sales_key'] = $name;
    $_SESSION['prosc_sales_date'] = $dated;
    $_SESSION['prosc_sales_name'] = $byname ?: $debtor->name;
    $_SESSION['prosc_sales_phone'] = $bphone ?: $debtor->phone;
    $_SESSION['prosc_sales_state'] = $_SESSION['prosc_warehouse'];
    $data = ['message'=>''];
    if ($token == '') $data = ['message'=>$this->pager('dump'), 'extra'=>'Starting Sale'];
    echo (json_encode(array_merge($data, ['status'=>'success'])));
  }

  // function navigate($dir)
  // {
  //   $client = $this->session->userdata('prosc_sales_client');
  //   if (($dir != 'next' && $dir != 'prev') OR !$this->salex_model->_get_client($client)) return false;

  //   $chip = $dir == 'prev' ? '-1 days' : '+1 day';
  //   $dated = $this->session->userdata('prosc_sales_date');
  //   $dated = date('Y-m-d', strtotime($chip, strtotime($dated)));
  //   $details = $this->salex_model->_get(['date'=>$dated, 'customer'=>$client]);
  //   $last = $this->salex_model->_count() + 1001;
  //   $last = str_repeat('0', 7 - strlen($last)) . $last;
  //   return $this->check_detail($details->voucher ?? $last, $dated, $client);
  // }

  function check()
  {
    $item = trim($this->input->post('item'));
    $qty  = trim($this->input->post('quantity'));
    $order_type =  '';
    
    (trim($this->input->post('order_type')) == '')? $order_type = 0 : $order_type = 2 ;
    
    // var_dump($order_type);
    if ($item == '' OR !is_numeric($qty)) return false;
    if ($qty < 1) return -1;

    $price = $this->salex_model->_get_price($item, 'a577f519b58b533e400cf28a7464576babf92121');
    
    return [
      'date'                 => $_SESSION['prosc_sales_date'],
      'voucher'              => $_SESSION['prosc_sales_key'],
      'price'                => $price[0],
      'cost_price'           => $price[1],
      'item'                 => $item,
      'type'                 => $order_type,
      'total'                => $qty * $price[0],
      'total_cost_price'     => $qty * $price[1],
      'customer_name' => $_SESSION['prosc_sales_name'],
      'customer_phone' => $_SESSION['prosc_sales_phone'],
      'warehouse' => $_SESSION['prosc_sales_state'],
      'customer'  => md5('er_empty'),
      'quantity'  => $qty,
      'done_by'   => $_SESSION['prosc_user']
    ];
  }

  function create()
  {
    if ($this->session->userdata('prosc_sales_key') == '') return $this->check_detail();

    $data = $this->check();
    if ( ! $data) exit(json_encode(['status'=>'error', 'message'=>'Some fields were not properly filled']));
    if ($data == -1) exit(json_encode(['status'=>'error', 'message'=>'Quantity is invalid']));
    $check = ['date'=>$data['date'], 'item'=>$data['item'], 'customer'=>$data['customer'], 'voucher'=>$data['voucher']];
    if ($this->salex_model->_get($check)) exit(json_encode(['status'=>'error', 'message'=>'This entry already exists in the sale']));

    $this->inventory_model->_aggregate($_SESSION['prosc_warehouse']);

    if ( ! $this->salex_model->_evaluate($data['item'], $_SESSION['prosc_warehouse'], $data['quantity'])) 
      exit(json_encode(['status'=>'error', 'message'=>'Insufficient Stock']));
    
    if ( ! $this->salex_model->_post($data))
      exit(json_encode(['status'=>'error', 'message'=>'An unknown error occured and the insertion operation failed']));

    echo (json_encode(['status'=>'success', 'message'=>$this->pager('dump')]));
  }

  function update()
  {
    $action = $this->input->get('action');
    if ($action != '') exit($this->$action());
    $target = $this->salex_model->_get(['token'=>trim($this->input->get('target'))]);
    if ( ! $target) exit(json_encode(['status'=>'error', 'message'=>'A fatal error has occured. <br />Please refresh and try again']));
    $data = $this->check(); 
    if ( ! $data) exit(json_encode(['status'=>'error', 'message'=>'Some fields were not properly filled']));
    if ($data == -1) exit(json_encode(['status'=>'error', 'message'=>'Quantity is invalid']));
    $check = ['token !='=>$target->token, 'date'=>$data['date'], 'item'=>$data['item'], 'customer'=>$data['customer'], 'voucher'=>$data['voucher']];
    if ($this->salex_model->_get($check)) exit(json_encode(['status'=>'error', 'message'=>'This entry already exists in the sale']));

    $this->inventory_model->_aggregate($_SESSION['prosc_warehouse']);
    $quantity = $data['quantity'] - $target->quantity;
    if ( ! $this->salex_model->_evaluate($data['item'], $_SESSION['prosc_warehouse'], $quantity)) 
      exit(json_encode(['status'=>'error', 'message'=>'Insufficient Stock']));

    if ( ! $this->salex_model->_put($data, $target->token))
      exit(json_encode(['status'=>'error', 'message'=>'An unknown error occured and the update failed']));
    
    echo (json_encode(['status'=>'success', 'message'=>$this->pager('dump')]));
  }

  function open()
  {
    $target = trim($this->input->get('target'));
    $sale = $this->salex_model->_get(['token'=>$target]);
    if ( ! $sale) exit(json_encode(['status'=>'error', 'message'=>'An unknown error occured while trying to open']));

    return $this->check_detail($sale->voucher, $sale->date, $sale->customer, $sale->customer_name, $sale->customer_phone);
  }

  function mod()
  {
    $dated  = trim($this->input->post('dated'));
    $client = md5('er_empty');
    $byname = trim($this->input->post('name'));
    $bphone = trim($this->input->post('phone'));

    if ($dated == '' OR $client == '' OR $byname == '' OR $bphone == '') exit(json_encode(['status'=>'error', 'message'=>'Fill in all fields']));
    if ($dated > date('Y-m-d')) exit(json_encode(['status'=>'error', 'message'=>'Invalid date supplied']));
    $target = $this->input->get('target');
    if ( ! $this->salex_model->_get(['voucher'=>$target])) exit(json_encode(['status'=>'error', 'message'=>'A fatal error has occured. <br />Please refresh and try again']));
    $check = ['date'=>$dated, 'customer'=>$client, 'customer_name'=>$byname, 'customer_phone'=>$bphone];
    if ($this->salex_model->_get($check)) exit(json_encode(['status'=>'error', 'message'=>'This Sale already exist']));
    $debtor = $this->salex_model->_get_client($client);
    if ( ! $debtor) exit(json_encode(['status'=>'error', 'message'=>'Request failed']));

    $data['date'] = $dated;
    $data['customer'] = $client;
    $data['customer_name'] = $byname ?: $debtor->name;
    $data['customer_phone'] = $bphone ?: $debtor->phone;
    $this->salex_model->_put($data, ['voucher'=>$target]);
    echo (json_encode(['status'=>'success', 'message'=>$this->pager('dump')]));
  }

  function close()
  {
    $this->salex_model->_close();
    $this->session->unset_userdata('prosc_sales_key');
    echo (json_encode(['status'=>'success', 'message'=>'']));
  }

  function delete()
  {
    $token = 'token';
    $target = trim($this->input->get('token'));
    $item = $this->salex_model->_get(['token'=>$target]);
    if ( ! $item) {
      $token = 'voucher';
      $target = trim($this->input->get('voucher'));
      $item = $this->salex_model->_get(['voucher'=>$target]);
    }

    if ( ! $item) exit(json_encode(['status'=>'error', 'message'=>'A fatal error has occured. <br />Please refresh and try again']));
    if(trim($this->input->get('tok')) == 'update'){
        $this->salex_model->_putx($target, 'voucher'); 
        $this->session->set_flashdata('prosc_message', 'Sales Order Completed');
    }else{
        $this->salex_model->_drop($target, 'voucher'); 
        $this->session->set_flashdata('prosc_message', 'Record Deleted');
    }

    echo (json_encode(['status'=>'success', 'message'=>$this->pager('dump')]));
    $this->inventory_model->_aggregate($_SESSION['prosc_warehouse']);
  }

  function popup()
  {
    $this->session->unset_userdata('prosc_sales_key');
    if ( ! $this->input->is_ajax_request()) {
      $pagedata['heading'] = 'Hold on!';
      $pagedata['message'] = 'The page you tried to reach, restricts direct access.';
      $this->load->view('errors/html/error_404', $pagedata);
    }
    $target = $this->input->get('token');
    $last = $this->salex_model->_count() + 1001;
    $last = str_repeat('0', 7 - strlen($last)) . $last;
    if (!isset($_SESSION['prosc_admin'])) $this->db->where('warehouse', $_SESSION['prosc_warehouse']);
    $clients = $this->salex_model->_get_clients();
    $data['data']    = [];
    $data['voucher'] = $last;
    $data['clients'] = $clients;
    $file = $this->salex_model->_get(['token'=>$target]);
    if ($file) $data['data'] = $file;
    $this->load->view('popups/admin/sale', $data);
  }
}
