<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Supply_products_control extends CI_Controller
{

    public function __construct()
    {
        parent::__construct();
        if ($this->session->userdata('prosc_admin') !== true)
            redirect(base_url(), 'refresh');

        $this->load->model('supply_products_model');
        $this->load->model('dashboard_model');
        $this->load->model('shipping_settings_model');
    }

    public function index()
    {
        $this->pager();
    }

    function pager($tmp = '')
    {
        if (!$this->input->is_ajax_request() && strstr(current_url(), '.aspx') == '') redirect(base_url('not_found.aspx'), 'refresh');
        $cats = $this->supply_products_model->_get_cats() ?: [];
        $items = $this->supply_products_model->_get_items() ?: [];
        $dated = $this->session->userdata('prosc_supply_date');
        $dated = ($dated == '' or $dated == date('Y-m-d')) ? date('Y-m-d') : $dated;
        $data  = $this->supply_products_model->_get_all(['er_supply.date' => $dated]) ?: [];
        $totals = $this->supply_products_model->_get_sums() ?: [];
        $country = $this->shipping_settings_model->_get_country();

        if ($country->value === 'Nigeria') {
            $states = $this->shipping_settings_model->_get_nigeria() ?: [];
        } else {
            $states = $this->shipping_settings_model->_get_uk() ?: [];
        }
        $currency = $this->dashboard_model->_get_currency() ?: "";
        $outlets = $this->supply_products_model->_get_outlets('all') ?: [];
        if ($tmp == '') $this->load->view('default', ['page' => 'supply_products', 'data' => $data, 'cats' => $cats, 'outlets' => $outlets, 'currency' => $currency, 'totals' => $totals, 'states' => $states, 'items' => $items, 'tmp' => $tmp]);
        else return $this->load->view('admin/supply_products', ['data' => $data, 'cats' => $cats, 'outlets' => $outlets, 'totals' => $totals, 'currency' => $currency, 'states' => $states, 'items' => $items, 'tmp' => $tmp], true);
    }
    function check_detail($token = '', $dated = '', $outlet = '')
    {
        $name   = $token ?: trim($this->input->post('autogen'));
        $dated  = $dated ?: trim($this->input->post('dated'));
        $outlet = $outlet ?: trim($this->input->post('outlet'));

        $store = $this->supply_products_model->_get_outlet($outlet);
        if ($name == '' or $dated == '' or $outlet == '') exit(json_encode(['status' => 'error', 'message' => 'Fill in all fields']));
        if ($dated > date('Y-m-d')) exit(json_encode(['status' => 'error', 'message' => 'Invalid date supplied']));
        if (!$store) exit(json_encode(['status' => 'error', 'message' => 'Request Error']));
        if ($token == '' && $this->supply_products_model->_get_keyed($name)) exit(json_encode(['status' => 'error', 'message' => 'Request Error']));
        $check = ['date' => $dated, 'outlet' => $outlet];
        if ($token == '' && $this->supply_products_model->_get($check)) exit(json_encode(['status' => 'error', 'message' => 'This Supply already exist']));

        if ($token != '') $name = $token;
        $_SESSION['prosc_supply_key'] = $name;
        $_SESSION['prosc_supply_date'] = $dated;
        $_SESSION['prosc_supply_name'] = $store->name;
        $_SESSION['prosc_supply_state'] = $store->state;
        $_SESSION['prosc_supply_outlet'] = $outlet;
        $data = ['message' => ''];
        if ($token == '') $data = ['message' => $this->pager('dump'), 'extra' => 'Supply Started'];
        echo (json_encode(array_merge($data, ['status' => 'success'])));
    }

    function navigate($dir)
    {
        $outlet = $this->session->userdata('prosc_supply_outlet');
        if (($dir != 'next' && $dir != 'prev')) return false;
        // if (($dir != 'next' && $dir != 'prev') OR !$this->supply_products_model->_get_outlet($outlet)) return false;

        $chip = $dir == 'prev' ? '-1 days' : '+1 day';
        $dated = $this->session->userdata('prosc_supply_date') ?? date('Y-m-d');
        $dated = date('Y-m-d', strtotime($chip, strtotime($dated)));
        // $details = $this->supply_products_model->_get(['date'=>$dated, 'outlet'=>$outlet]);
        // $last = $this->supply_products_model->_count() + 1001;
        // $last = str_repeat('0', 7 - strlen($last)) . $last;
        // return $this->check_detail($details->voucher ?? $last, $dated, $outlet);
        $this->session->set_userdata('prosc_supply_date', $dated);
        echo json_encode(['status' => 'success', 'message' => '']);
    }

    function check()
    {
        $item = trim($this->input->post('item'));
        $qty  = trim($this->input->post('quantity'));
        if ($item == '' or !is_numeric($qty)) return false;
        if ($qty < 1) return -1;

        $price = $this->supply_products_model->_get_price($item, 'a577f519b58b533e400cf28a7464576babf92121');
        return [
            'date'     => $_SESSION['prosc_supply_date'],
            'voucher'  => $_SESSION['prosc_supply_key'],
            'price'    => $price,
            'item'     => $item,
            'total'    => $qty * $price,
            'state'    => $_SESSION['prosc_supply_state'],
            'outlet'   => $_SESSION['prosc_supply_outlet'],
            'quantity' => $qty,
            'done_by'  => $_SESSION['prosc_user']
        ];
    }

    function create()
    {
        if ($this->session->userdata('prosc_supply_key') == '') return $this->check_detail();

        $data = $this->check();
        if ($data == -1) exit(json_encode(['status' => 'error', 'message' => 'Quantity is invalid']));
        if (!$data) exit(json_encode(['status' => 'error', 'message' => 'Some fields were not properly filled']));

        $check = ['date' => $data['date'], 'item' => $data['item'], 'outlet' => $data['outlet'], 'voucher' => $data['voucher']];
        if ($this->supply_products_model->_get($check)) exit(json_encode(['status' => 'error', 'message' => 'This entry already exists in the supply']));

        if (!$this->supply_products_model->_post($data))
            exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured and the insertion operation failed']));

        echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
    }

    function update()
    {
        $action = $this->input->get('action');
        if ($action != '') exit($this->$action());
        $target = trim($this->input->get('target'));
        if (!$this->supply_products_model->_get(['token' => $target])) exit(json_encode(['status' => 'error', 'message' => 'A fatal error has occured. <br />Please refresh and try again']));
        $data = $this->check();
        if (!$data) exit(json_encode(['status' => 'error', 'message' => 'Some fields were not properly filled']));
        if ($data == -1) exit(json_encode(['status' => 'error', 'message' => 'Quantity is invalid']));
        $check = ['token !=' => $target, 'date' => $data['date'], 'item' => $data['item'], 'outlet' => $data['outlet'], 'voucher' => $data['voucher']];
        if ($this->supply_products_model->_get($check)) exit(json_encode(['status' => 'error', 'message' => 'This entry already exists in the supply']));

        if (!$this->supply_products_model->_put($data, $target))
            exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured and the update failed']));

        echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
    }

    function open()
    {
        $target = trim($this->input->get('target'));
        $supply = $this->supply_products_model->_get(['token' => $target]);
        if (!$supply) exit(json_encode(['status' => 'error', 'message' => 'An unknown error occured while trying to open']));

        return $this->check_detail($supply->voucher, $supply->date, $supply->outlet);
    }

    function mod()
    {
        $dated  = trim($this->input->post('dated'));
        $outlet = trim($this->input->post('outlet'));

        if ($dated == '' or $outlet == '') exit(json_encode(['status' => 'error', 'message' => 'Fill in all fields']));
        if ($dated > date('Y-m-d')) exit(json_encode(['status' => 'error', 'message' => 'Invalid date supplied']));
        $target = $this->input->get('target');
        if (!$this->supply_products_model->_get(['voucher' => $target])) exit(json_encode(['status' => 'error', 'message' => 'A fatal error has occured. <br />Please refresh and try again']));
        $check = ['date' => $dated, 'outlet' => $outlet];
        if ($this->supply_products_model->_get($check)) exit(json_encode(['status' => 'error', 'message' => 'This Supply already exist']));

        $data['date'] = $dated;
        $data['outlet'] = $outlet;
        $this->supply_products_model->_put($data, ['voucher' => $target]);
        echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
    }

    function close()
    {
        $this->supply_products_model->_close();
        $this->session->unset_userdata('prosc_supply_key');
        echo (json_encode(['status' => 'success', 'message' => '']));
    }

    function delete()
    {
        $token = 'token';
        $target = trim($this->input->get('token'));
        $item = $this->supply_products_model->_get(['token' => $target]);
        if (!$item) {
            $token = 'voucher';
            $target = trim($this->input->get('voucher'));
            $item = $this->supply_products_model->_get(['voucher' => $target]);
        }

        if (!$item) exit(json_encode(['status' => 'error', 'message' => 'A fatal error has occured. <br />Please refresh and try again']));

        $this->supply_products_model->_drop($target, $token);
        $this->session->set_flashdata('prosc_message', 'Record Deleted');
        echo (json_encode(['status' => 'success', 'message' => $this->pager('dump')]));
    }

    function popup()
    {
        $this->session->unset_userdata('prosc_supply_key');
        if (!$this->input->is_ajax_request()) {
            $pagedata['heading'] = 'Hold on!';
            $pagedata['message'] = 'The page you tried to reach, restricts direct access.';
            $this->load->view('errors/html/error_404', $pagedata);
        }
        $target = $this->input->get('token');
        $last = $this->supply_products_model->_count() + 10010001;
        $outlets = $this->supply_products_model->_get_outlets();
        $data['data']    = [];
        $data['voucher'] = $last;
        $data['outlets'] = $outlets;
        $file = $this->supply_products_model->_get(['token' => $target]);
        if ($file) $data['data'] = $file;
        $this->load->view('popups/admin/supply_products', $data);
    }
}
