<?php
if (!defined('BASEPATH'))
    exit('No direct script access allowed');

class Dispatch_model extends CI_Model
{

    function __construct()
    {
        parent::__construct();
    }

    function _get($params)
    {
        $this->db->where('status', 0);
        // $users = $this->db->get_where('er_waybill_supply', $params);
        //check row not found
        $users = $this->db->get_where('er_waybill', $params);
        if (!($users->num_rows() > 0))  return false;
        return $users->result_array();
    }


    function _get_all($params = '')
    {
        if ($params != '') $this->db->where($params);
        $users = $this->db->get_where('er_waybill', ['status' => 0]);
        if (!($users->num_rows() > 0)) return false;
        return $users->result_array();
    }

    function _get_customer($id)
    {
        $this->db->select('name, address');
        $user = $this->db->get_where('er_customers', ['token' => $id]);
        if (!$user->num_rows() > 0) return '';
        return $user->row_array();
    }

    // get the items
    function _get_items($params = '')
    {
        $items = $this->db->get_where('er_items', ['status' => 0]);
        if (!($items->num_rows() > 0)) return false;
        return $items->result_array();
    }

    function _get_temps()
    {
        $data = [];
        $where = '';
        if (isset($_SESSION['prosc_way_update']))
            $where = "(`waybill`='' OR `waybill`='" . $_SESSION['prosc_way_token'] . "')";
        else $where = "`waybill`=''";

        $this->db->where("(`type`='1' OR (`type`='2' AND `paid`='1'))");
        $this->db->where($where)->group_by('voucher');
        $sales = $this->db->get_where('er_sales', ['status' => 0]);
        if (($sales->num_rows() > 0)) {
            foreach ($sales->result_array() as $row) {
                $k = ['voucher' => $row['voucher'], 'status' => 0];
                $t = $row['type'] == 2 ? 'XC-' : 'XS-';
                $cs = $this->_get_customer($row['customer']);
                $data[] = [
                    'token'   => $row['token'],
                    'date'    => strtotime($row['date']),
                    'voucher' => $t . $row['voucher'],
                    'station' => $row['customer_name'] ?: $cs['name'],
                    'waybill' => $row['waybill'],
                    'address' => $cs['address'],
                    'items'   => $this->_count('sales', $k),
                    'total'   => $this->_get_sums('sales', $k)
                ];
            }
        }
        $this->db->where($where)->group_by('voucher');
        $sales = $this->db->get_where('er_supply', ['status' => 0]);
        if (($sales->num_rows() > 0)) {
            foreach ($sales->result_array() as $row) {
                $k = ['voucher' => $row['voucher'], 'status' => 0];
                $cs = $this->_get_outlet($row['outlet']);
                $data[] = [
                    'token'   => $row['token'],
                    'date'    => strtotime($row['date']),
                    'voucher' => $row['voucher'],
                    'station' => $cs['name'],
                    'address' => $cs['address'],
                    'waybill' => $row['waybill'],
                    'items'   => $this->_count('supply', $k),
                    'total'   => $this->_get_sums('supply', $k)
                ];
            }
        }
        if (count($data) == 0) return false;
        array_multisort(array_column($data, 'date'), SORT_DESC, $data);
        return $data;
    }

    function _get_sums($table, $params)
    {
        $sum = 0;
        $table = 'er_' . $table;
        $this->db->select("SUM(total) as total");
        $this->db->group_by('voucher');
        $sql = $this->db->get_where($table, $params);
        if ($sql->num_rows() > 0) {
            $result = $sql->result_array();
            foreach ($result as $row) {
                $sum = $row['total'];
            }
        }
        return $sum;
    }

    function _get_outlet($token)
    {
        $this->db->select('name, address');
        $this->db->where('status', 0);
        $outlet = $this->db->get_where('er_warehouse', ['token' => $token]);
        if (!($outlet->num_rows() > 0)) return false;
        return $outlet->row_array();
    }

    function _get_verifyItem($params, $table)
    {
        $data = []; // Initialize the $data array here

        if ($table == 'er_sales') {
            $items = $this->db->get_where('er_sales', $params);
        } else {
            $items = $this->db->get_where('er_supply', $params);
        }

        if ($items->num_rows() > 0) {
            foreach ($items->result_array() as $row) {
                if ($table == 'er_sales') {
                    $cs = $this->_get_customer($row['customer']);
                    $customer = $row['customer_name'] ?: $cs['name'];
                } else {
                    $cs = $this->_get_outlet($row['outlet']);
                    $customer = $cs['name'];
                }

                $data[] = [
                    'sales_id'        => $row['token'],
                    'order_id'        => $row['voucher'],
                    'waybill'         => '',
                    'address'         => $table == 'er_sales' ? $this->_get_sales_xtal(['voucher' => $row['voucher']]) : "Xcel supply",
                    'customer'        => $customer,
                    'item'            => $row['item'],
                    'quantity'        => $row['quantity'],
                    'total'        => $row['total'],
                    'shipment_status' => 'Shipment Created',
                    'status'          => 0,
                ];
            }
        }

        return $data;
    }


    function _get_sales($params, $return_all = false)
    {
        $this->db->where('status', 0);

        if ($return_all) {
            $user = $this->db->get_where('er_sales', ['order_status' => 'Shipment Initialized']);
            if (!($user->num_rows() > 0)) return false;
            return $user->result_array();
        } else {
            $user = $this->db->get_where('er_sales', $params);
            if (!($user->num_rows() > 0)) return false;
            return $user->row();
        }
    }


    function _get_supply($params, $return_all = false)
    {
        $this->db->where('status', 0);

        if ($return_all) {
            $user = $this->db->get_where('er_supply', ['order_status' => 'Shipment Initialized']);
            if (!($user->num_rows() > 0)) return false;
            return $user->result_array();
        } else {
            $user = $this->db->get_where('er_supply', $params);
            if (!($user->num_rows() > 0)) return false;
            return $user->row();
        }
    }

    function _get_states()
    {
        $users = $this->db->get_where('er_nigeria_state', ['status' => 0]);
        if (!($users->num_rows() > 0)) return false;
        return $users->result_array();
    }

    //automatical change


    function _put($params, $target)
    {
        $this->db->where('token', $target);
        $user = $this->db->update('er_waybill', $params);
        if (!$user) return false;
        return true;
    }

    function _update($target)
    {
        $this->db->where('token', $target);
        $use = $this->db->update("er_waybill", ['delivery' => '1']);
        return true;
    }

    // function _item($table, $filter = '')
    // {
    //   $table = 'er_' . $table;

    //   // Initialize the query builder
    //   $this->db->from($table);

    //   // Add the filter condition
    //   if (is_array($filter)) {
    //     $this->db->where($filter);
    //   }

    //   // Add additional condition where waybill = ""
    //   $this->db->where('waybill', '');

    //   // Execute the query
    //   $query = $this->db->get();

    //   if ($query->num_rows() > 0) {
    //     // Fetch all rows and extract the "item" column values into an array
    //     $items = $query->result_array();
    //     $itemValues = array_column($items, 'item');

    //     return $itemValues;
    //   } else {
    //     // If no rows are found, return an empty array
    //     return [];
    //   }
    // }


    function _get_sales_xtal($params, $all = false)
    {
        if ($all) {
            $this->db->where('status', 0);
            $user = $this->db->get_where('er_sales_extra', $params);
            if (!($user->num_rows() > 0)) return false;
            return $user->row();
        } else {
            $this->db->select('shipping');
            $debtor = $this->db->get_where('er_sales_extra', $params);

            if ($debtor->num_rows() > 0) {
                $row = $debtor->row_array();
                $dataArray = json_decode($row['shipping'], true);
                return $dataArray['address'];
            }

            return '';
        }
    }

    function _count($table, $filter = '')
    {
        $table = 'er_' . $table;
        if (is_array($filter)) $this->db->where($filter);
        return $this->db->count_all_results($table);
    }

    function _mod($table, $params, $data)
    {
        $this->db->where($params);
        if (!$this->db->update($table, $data)) return false;
        return true;
    }

    function _mod_shipping($params, $data)
    {
        $this->db->where($params);
        if (!$this->db->update('er_shipping', $data)) return false;
        return true;
    }

    function _post($data)
    {
        $hash = base64_encode(random_bytes(32) . date('Hisa'));
        $hash = strtr($hash, '+/', '-_');
        $hash = rtrim($hash, '=');
        $data['token'] = $hash;
        $data['done_by'] = $_SESSION['prosc_user'];
        if (!$this->db->insert('er_waybill', $data)) return false;
        return $data['token'];
    }

    function _get_token()
    {
        $hash = base64_encode(random_bytes(32) . date('Hisa'));
        $hash = strtr($hash, '+/', '-_');
        $hash = rtrim($hash, '=');

        return $hash;
    }

    function _post_shipping($data, $table)
    {
        $hash = base64_encode(random_bytes(32) . date('Hisa'));
        $hash = strtr($hash, '+/', '-_');
        $hash = rtrim($hash, '=');
        $data['token'] = $hash;
        $this->db->where('token', $data['sales_id']);
        if (!$this->db->update($table, ['order_status' => 'Shipment Created'])) return false;
        if (!$this->db->insert('er_shipping', $data)) return false;
        return $data['token'];
    }


    // function _post_supply($data)
    // {
    //   if ( ! $this->db->insert_batch('er_waybill_supply', $data)) return false;
    //   return true;
    // }

    // function _close()
    // {
    //   // $code = $_SESSION['prosc_supply_key'];
    //   // $date = $_SESSION['prosc_supply_date'];
    //   // $this->db->where(['voucher'=>$code, 'date'=>$date]);
    //   // $this->db->update('er_supply', ['save'=>'1']);
    //   // return true;
    // }


    function _drop($target)
    {
        $this->db->where('token', $target);
        $this->db->update('er_waybill', ['status' => '1']);
    }
}
