<?php
if (!defined('BASEPATH'))
  exit('No direct script access allowed');

class Product_items_model extends CI_Model
{

  function __construct()
  {
    parent::__construct();
  }

  function _get($params)
  {
    $this->db->where('status', 0);
    $item = $this->db->get_where('er_items', $params);
    if (!($item->num_rows() > 0)) return false;
    return $item->row();
  }

  function _get_min()
  {
    $stock = $this->db->get_where('er_settings', ['name' => 'stock_limit']);
    if (!($stock->num_rows() > 0)) return false;
    return $stock->row('value');
  }

  function _get_all($params = '')
  {
    $this->db->order_by('created_at ASC');
    if ($params != '') $this->db->where($params);
    $items = $this->db->get_where('er_items', ['status' => 0]);
    if (!($items->num_rows() > 0)) return false;
    return $items->result_array();
  }

  function _has_child_cat($params)
  {
    $this->db->where('status', 0);
    $cat = $this->db->get_where('er_categories', $params);
    return ($cat->num_rows() > 0) ? true : false;
  }

  function _get_categories()
  {
    $this->db->order_by('parent');
    $cats = $this->db->get_where('er_categories', ['status' => 0]);
    if (!($cats->num_rows() > 0)) return false;
    $name = $data = [];
    foreach ($cats->result_array() as $row) {
      if ($this->_has_child_cat(['parent' => $row['token']])) {
        $name[$row['token']] = $row['name'];
        continue;
      }
      if (array_key_exists($row['parent'], $name)) $row['name'] .= ' &middot; (' . $name[$row['parent']] . ')';
      $data[] = $row;
    }
    return $data;
  }

  function _get_states()
  {
    $states = $this->db->get_where('er_nigeria_state', ['status' => 0]);
    if (!($states->num_rows() > 0)) return false;
    return $states->result_array();
  }

  function _put($params, $target)
  {
    $this->db->where('token', $target);

    // Update the first table
    $item = $this->db->update('er_items', $params);
    if (!$item) {
      return false;
    }

    // Update the second table 
    $this->db->where('token', $target);
    $onlineItem = $this->db->update('er_online_warehouse', $params);

    // Check if the update for the second table was successful
    if (!$onlineItem) {
      // If the second update fails, you might want to roll back the first update
      $this->db->where('token', $target);
      $this->db->update('er_items', $params);
      return false;
    }

    return true;
  }


  function _count()
  {
    return $this->db->count_all_results('er_items');
  }

  function _set_min($value)
  {
    $this->db->where(['name' => 'stock_limit']);
    $stock = $this->db->update('er_settings', ['value' => $value]);
    if (!$stock) return false;
    return true;
  }

  function _post($data)
  {

    $hash = base64_encode(random_bytes(32) . date('Hisa'));
    $hash = strtr($hash, '+/', '-_');
    $hash = rtrim($hash, '=');
    $last = $this->_count() + 1;
    $uuid = str_repeat('0', 3 - strlen($last)) . $last;
    $data['token'] = $hash;
    // Insert into the first table
    if (!$this->db->insert('er_items', $data)) {
      return false;
    }
    // Insert into the second table
    if (!$this->db->insert('er_online_warehouse', $data)) {
      // If the second insert fails, you might want to roll back the first insert
      $this->db->where('token', $data['token']);
      $this->db->delete('er_items');
      return false;
    }

    return $data['token'];
  }

  function _drop($target)
  {
    $this->db->where('item', $target);
    $this->db->update('er_outlet_store', ['status' => '1']);
    $this->db->where('token', $target);
    $this->db->update('er_items', ['status' => '1']);
  }
}
