<?php
if (!defined('BASEPATH'))
    exit('No direct script access allowed');

class Provision_model extends CI_Model
{

    function __construct()
    {
        parent::__construct();
    }

    function _get($params)
    {
        $this->db->where('status', 0);
        $user = $this->db->get_where('er_supply', $params);
        if (!($user->num_rows() > 0)) return false;
        return $user->row();
    }

    function _get_keyed($token)
    {
        $this->db->where('status', 0);
        $supply = $this->db->get_where('er_supply', ['voucher' => $token]);
        if (!($supply->num_rows() > 0)) return false;
        return true;
    }

    function _get_outlet($token)
    {
        $this->db->where('status', 0);
        $outlet = $this->db->get_where('er_warehouse', ['token' => $token]);
        if (!($outlet->num_rows() > 0)) return false;
        return $outlet->row();
    }

    function _get_all($params = '')
    {
        if (isset($_SESSION['prosc_supply_key'])) $this->db->where('er_supply.voucher', $_SESSION['prosc_supply_key']);
        if (!isset($_SESSION['prosc_supply_x']) && $params != '') $this->db->where($params);
        $this->db->order_by('er_supply.voucher DESC');
        if ($this->session->userdata('prosc_supply_key') == '' && !isset($_SESSION['prosc_supply_x'])) {
            $this->db->group_by('er_supply.voucher');
            $this->db->select('er_supply.token, er_supply.voucher, er_supply.outlet, er_supply.approve, er_supply.created_at, GROUP_CONCAT(er_items.name) as items, er_supply.date');
            $this->db->from('er_supply')->join('er_items', 'er_supply.item = er_items.token');
        } else {
            $this->db->select('*')->from('er_supply');
        }
        if (isset($_SESSION['prosc_supply_x']))
            $this->db->where("`er_supply.date` BETWEEN '" . $_SESSION['prosc_supply_x'] . "' AND '" . $_SESSION['prosc_supply_y'] . "'");

        $users = $this->db->get();
        if (!($users->num_rows() > 0)) return false;
        return $users->result_array();
    }

    function _get_states()
    {
        $users = $this->db->get_where('er_state', ['status' => 0]);
        if (!($users->num_rows() > 0)) return false;
        return $users->result_array();
    }

    function _get_outlets($status = '')
    {
        $this->db->order_by('type ASC, name ASC');
        if ($status != 'all') $this->db->where('status', '0');
        $outlets = $this->db->get('er_warehouse');
        if (!($outlets->num_rows() > 0)) return false;
        return $outlets->result_array();
    }

    function _get_sums()
    {
        $data = [];
        $this->db->select("voucher, SUM(total) as total");
        $this->db->group_by('voucher');
        $sql = $this->db->get_where('er_supply', ['status' => 0]);
        if ($sql->num_rows() > 0) {
            $result = $sql->result_array();
            foreach ($result as $row) {
                $data[$row['voucher']] = $row['total'];
            }
        }
        return $data;
    }

    function _get_price($item, $state)
    {
        $items = $this->db->get_where('er_items', ['token' => $item]);
        if (!($items->num_rows() > 0)) return 0;
        $price = json_decode($items->row('price'), true);
        return $price[$state][0];
    }

    function _get_items()
    {
        $this->db->order_by('name ASC');
        $items = $this->db->get_where('er_items', ['status' => 0]);
        if (!($items->num_rows() > 0)) return false;
        $items = $items->result_array();
        $item  = [];
        foreach ($items as $row) {
            $k = json_decode($row['price'], true) ?? [];

            $k = $k['a577f519b58b533e400cf28a7464576babf92121'] ?? 0;
            $item[] = ['token' => $row['token'], 'name' => $row['name'], 'price' => number_format($k[0], 1), 'cost_price' => number_format($k[1], 1), 'category' => $row['category'], 'status' => $row['status']];
            // $k = $k['a577f519b58b533e400cf28a7464576babf92121'][0] ?? 0;
            // $item[] = ['token'=>$row['token'], 'name'=>$row['name'], 'price'=>number_format($k, 1), 'category'=>$row['category'], 'status'=>$row['status']];
        }
        return $item;
    }

    function _get_cats()
    {
        $items = $this->db->get_where('er_categories', ['status' => 0]);
        if (!($items->num_rows() > 0)) return false;
        return $items->result_array();
    }

    function _close()
    {
        $code = $_SESSION['prosc_supply_key'];
        $date = $_SESSION['prosc_supply_date'];
        $this->db->where(['voucher' => $code, 'date' => $date]);
        $this->db->update('er_supply', ['save' => '1']);
        return true;
    }

    function _put($params, $target)
    {
        if (is_array($target)) $this->db->where($target);
        else $this->db->where('token', $target);
        $user = $this->db->update('er_supply', $params);
        if (!$user) return false;
        return true;
    }

    function _count($filter = '')
    {
        if (is_array($filter)) $this->db->where($filter);
        else $this->db->group_by('voucher');
        return $this->db->count_all_results('er_supply');
    }

    function _post($data)
    {
        $hash = base64_encode(random_bytes(32) . date('Hisa'));
        $hash = strtr($hash, '+/', '-_');
        $hash = rtrim($hash, '=');
        $uuid = $this->_count() + 10010001;
        $data['token'] = $hash;
        if (!$this->db->insert('er_supply', $data)) return false;
        return $data['token'];
    }

    function _drop($target, $col = 'token')
    {
        $this->db->where($col, $target);
        $this->db->update('er_supply', ['status' => '1']);
    }
}
