<?php
if (!defined('BASEPATH'))
  exit('No direct script access allowed');

class Waybill_model extends CI_Model
{

  function __construct()
  {
    parent::__construct();
  }

  function _get($params)
  {
    $this->db->where('status', 0);
    $bill = $this->db->get_where('er_waybill', $params);
    if (!($bill->num_rows() > 0)) return false;
    return $bill->row();
  }


  function _get_all()
  {
    $this->db->order_by('created_at DESC');
    $waybill = $this->db->get_where('er_waybill', ['status' => 0]);
    if (!$waybill->num_rows() > 0) return '';
    return $waybill->result_array();
  }

  function _get_customer($id)
  {
    $this->db->select('name, address');
    $debtor = $this->db->get_where('er_customers', ['token' => $id]);
    if (!$debtor->num_rows() > 0) return '';
    return $debtor->row_array();
  }


  function _get_temps($token = '')
  {
    $data = [];
    $where = '';
    if ($token != '')
      $where = "(`waybill`='' OR `waybill`='" . $token . "')";
    else $where = "`waybill`=''";

    $this->db->where("(`type`='1' OR (`type`='2' AND `paid`='1'))");
    $this->db->where($where)->group_by('voucher');
    $sales = $this->db->get_where('er_sales', ['status' => 0]);
    if (($sales->num_rows() > 0)) {
      foreach ($sales->result_array() as $row) {
        $k = ['voucher' => $row['voucher'], 'status' => 0];
        $cs = $this->_get_customer($row['customer']);
        $csx = $this->_get_sales_xtal($row['voucher']);
        $data[] = [
          'token'   => $row['token'],
          'date'    => strtotime($row['date']),
          'voucher' => $row['voucher'],
          'type' => $row['type'],
          'station' => $row['customer_name'] ?: $cs['name'],
          'waybill' => $row['waybill'],
          'quantity' =>  $row['quantity'],
          'address' => $cs['address'] ?:   $csx,
          'item' => $this->_item('sales', $k),
          'items'   => $this->_count('sales', $k),
          'total'   => $this->_get_sums('sales', $k)
        ];
      }
    }
    $this->db->where($where)->group_by('voucher');
    $sales = $this->db->get_where('er_supply', ['status' => 0]);
    if (($sales->num_rows() > 0)) {
      foreach ($sales->result_array() as $row) {
        $k = ['voucher' => $row['voucher'], 'status' => 0];
        $cs = $this->_get_outlet($row['outlet']);
        $data[] = [
          'token'   => $row['token'],
          'date'    => strtotime($row['date']),
          'voucher' => $row['voucher'],
          'station' => $cs['name'],
          'type' => 'supply',
          'address' => $cs['address'],
          'quantity' =>  $row['quantity'],
          'waybill' => $row['waybill'],
          'item' => $this->_item('supply', $k),
          'items'   => $this->_count('supply', $k),
          'total'   => $this->_get_sums('supply', $k)
        ];
      }
    }
    if (count($data) == 0) return false;
    array_multisort(array_column($data, 'date'), SORT_DESC, $data);
    return $data;
  }

  function _get_customers()
  {
    $this->db->where(['type' => 2]);
    $customers = $this->db->get_where('er_customers', ['status' => 0]);
    if (!$customers->num_rows() > 0) return false;
    return $customers->result_array();
  }

  function _get_states()
  {
    $states = $this->db->get_where('er_nigeria_state', ['status' => 0]);
    if (!($states->num_rows() > 0)) return false;
    return $states->result_array();
  }

  function _get_sums($table, $params)
  {
    $sum = 0;
    $table = 'er_' . $table;
    $this->db->select("SUM(total) as total");
    $this->db->group_by('voucher');
    $sql = $this->db->get_where($table, $params);
    if ($sql->num_rows() > 0) {
      $result = $sql->result_array();
      foreach ($result as $row) {
        $sum = $row['total'];
      }
    }
    return $sum;
  }

  function _get_outlet($token)
  {
    $this->db->select('name, address');
    $this->db->where('status', 0);
    $outlet = $this->db->get_where('er_warehouse', ['token' => $token]);
    if (!($outlet->num_rows() > 0)) return false;
    return $outlet->row_array();
  }

  function _get_sales($params, $return_all = false)
  {
    $this->db->where('status', 0);

    if ($return_all) {
      $user = $this->db->get_where('er_sales', ['order_status' => 'Shipment Initialized']);
      if (!($user->num_rows() > 0)) return false;
      return $user->result_array();
    } else {
      $user = $this->db->get_where('er_sales', $params);
      if (!($user->num_rows() > 0)) return false;
      return $user->row();
    }
  }

  function _get_sales_xtal($params, $all = false)
  {
    if ($all) {
      $this->db->where('status', 0);
      $user = $this->db->get_where('er_sales_extra', $params);
      if (!($user->num_rows() > 0)) return false;
      return $user->row();
    } else {

      $this->db->select('shipping');
      $debtor = $this->db->get_where('er_sales_extra', $params);

      if ($debtor->num_rows() > 0) {
        $row = $debtor->row_array();
        $dataArray = json_decode($row['shipping'], true);
        return $dataArray['address'];
      }
      return '';
    }
  }

  function _mod($params, $target)
  {
    $this->db->where($params);
    if ($this->db->update('er_sales', $target)) {
      return true;
    } else {
      return false;
    }
  }

  function _mod_shipping($params, $target)
  {
    $this->db->where($params);
    $update = $this->db->update('er_shipping', $target);
    if (!$update) return false;
    return true;
  }

  function _put($params, $target)
  {
    $this->db->where('token', $target);
    $waybill = $this->db->update('er_waybill', $params);
    if (!$waybill) return false;
    return true;
  }

  function _count($table, $filter = '')
  {
    $table = 'er_' . $table;
    if (is_array($filter)) $this->db->where($filter);
    return $this->db->count_all_results($table);
  }

  function _item($table, $filter = '')
  {
    $table = 'er_' . $table;

    // Initialize the query builder
    $this->db->from($table);

    // Add the filter condition
    if (is_array($filter)) {
      $this->db->where($filter);
    }

    // Add additional condition where waybill = ""
    $this->db->where('waybill', '');

    // Execute the query
    $query = $this->db->get();

    if ($query->num_rows() > 0) {
      // Fetch all rows and extract the "item" column values into an array
      $items = $query->result_array();
      $itemValues = array_column($items, 'item');

      return $itemValues;
    } else {
      // If no rows are found, return an empty array
      return [];
    }
  }


  function _post_shipping($data, $table)
  {
    $hash = base64_encode(random_bytes(32) . date('Hisa'));
    $hash = strtr($hash, '+/', '-_');
    $hash = rtrim($hash, '=');
    $data['token'] = $hash;

    if ($table == 'er_sales') {
      $this->db->where('token', $data['sales_id']);
      if (!$this->db->update('er_sales', ['order_status' => 'Shipment Created'])) return false;
    }
    if (!$this->db->insert('er_shipping', $data)) return false;
    return $data['token'];
  }

  function _post($data)
  {
    $hash = base64_encode(random_bytes(32) . date('Hisa'));
    $hash = strtr($hash, '+/', '-_');
    $hash = rtrim($hash, '=');
    $data['token'] = $hash;
    $data['done_by'] = $_SESSION['prosc_user'];
    if (!$this->db->insert('er_waybill', $data)) return false;
    return $data['token'];
  }
}
