<?php
if (!defined('BASEPATH'))
    exit('No direct script access allowed');

class Kpi_model extends CI_Model
{

    function __construct()
    {
        parent::__construct();
        $this->load->dbforge();
    }

    function _get($params)
    {
        $this->db->where('status', 0);
        $kpi = $this->db->get_where('er_kpi', $params);
        if (!($kpi->num_rows() > 0)) return false;
        return $kpi->row();
    }

    function _get_staff_per($params)
    {
        if ($this->db->field_exists($params, 'er_staff_performance')) {
            return true;
        }
        return false;
    }

    function _get_outlets()
    {
        $this->db->order_by('type ASC, name ASC');
        $outlets = $this->db->get_where('er_warehouse', ['status' => 0, 'id !=' => 1]);
        if (!($outlets->num_rows() > 0)) return false;
        return $outlets->result_array();
    }


    function _get_all($params = '')
    {
        if ($params != '') $this->db->where($params);
        $kpi = $this->db->get_where('er_kpi', ['status' => 0]);
        if (!($kpi->num_rows() > 0)) return false;
        return $kpi->result_array();
    }

    function _has_child_cat($params)
    {
        $this->db->where('status', 0);
        $cat = $this->db->get_where('er_categories', $params);
        return ($cat->num_rows() > 0) ? true : false;
    }

    function _get_categories()
    {
        $this->db->order_by('parent');
        $cats = $this->db->get_where('er_categories', ['status' => 0]);
        if (!($cats->num_rows() > 0)) return false;
        $name = $data = [];
        foreach ($cats->result_array() as $row) {
            if ($this->_has_child_cat(['parent' => $row['token']])) {
                $name[$row['token']] = $row['name'];
                continue;
            }
            if (array_key_exists($row['parent'], $name)) $row['name'] .= ' &middot; (' . $name[$row['parent']] . ')';
            $data[] = $row;
        }
        return $data;
    }

    function _get_states()
    {
        $states = $this->db->get_where('er_state', ['status' => 0]);
        if (!($states->num_rows() > 0)) return false;
        return $states->result_array();
    }

    function _put($params, $target)
    {
        $this->db->where('token', $target);
        $item = $this->db->update('er_items', $params);
        if (!$item) return false;
        return true;
    }

    function _count()
    {
        return $this->db->count_all_results('er_items');
    }

    function _set_min($value)
    {
        $this->db->where(['name' => 'stock_limit']);
        $stock = $this->db->update('er_settings', ['value' => $value]);
        if (!$stock) return false;
        return true;
    }

    function _post($data)
    {
        $hash = base64_encode(random_bytes(32) . date('Hisa'));
        $hash = strtr($hash, '+/', '-_');
        $hash = rtrim($hash, '=');
        $data['token'] = $hash;
        $newField = array(
            $data['name'] => array(
                'type' => 'FLOAT',
                'after' => 'staff_id',
            ),
        );
        if (!$this->dbforge->add_column('er_staff_performance', $newField)) return false;
        if (!$this->db->insert('er_kpi', $data)) return false;
        return $data['token'];
    }

    function _drop($target)
    {
        $this->db->where('item', $target);
        $this->db->update('er_outlet_store', ['status' => '1']);
        $this->db->where('token', $target);
        $this->db->update('er_items', ['status' => '1']);
    }
}
